import sys
import matplotlib
matplotlib.use("Agg") 
import numpy as np
plt.ioff()


import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages

# ceres modules
import os
_pkg_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
_data_dir = os.path.join(_pkg_dir, 'instruments', 'data')
_xc_masks_dir = os.path.join(_pkg_dir, 'utils', 'data', 'xc_masks') + '/'

baryc_dir = ''
ephemeris = 'DEc403'

from ceres3.instruments import dupontutils
from ceres3.utils import continuum
from ceres3.utils import correlation
from ceres3.utils import globalutils as GLOBALutils
from ceres3.ext import Marsh

# other useful modules
import argparse
import ephem
import glob
from ceres3.utils import jplephem
import pickle
from astropy.io import fits as pyfits
import scipy
import scipy.interpolate
from scipy import optimize
from scipy import interpolate
from numpy import radians as rad

import statsmodels.api as sm
lowess = sm.nonparametric.lowess

# Recive input parameters
parser = argparse.ArgumentParser()
parser.add_argument('directorio')
parser.add_argument('-avoid_plot', action="store_true", default=False)
parser.add_argument('-dirout',default='default')
parser.add_argument('-do_class', action="store_true", default=False)
parser.add_argument('-just_extract', action="store_true", default=False)
parser.add_argument('-no_dark_sub',     action = "store_true", default = False)
parser.add_argument('-npools', default=1)
parser.add_argument('-oblaze',   default = 'same')
parser.add_argument('-ofind',    default = 'last')
parser.add_argument('-o2do',default='all')
parser.add_argument('-reffile',default='default')
parser.add_argument('-resolution',default='40000')
args = parser.parse_args()

dirin                = args.directorio
avoid_plot           = args.avoid_plot
dirout               = args.dirout
DoClass              = args.do_class
JustExtract          = args.just_extract
no_dark_substraction = args.no_dark_sub
npools               = int(args.npools)
object2do            = args.o2do
reffile              = args.reffile
stst                 = args.ofind
stblaze              = args.oblaze
resolution           = float(args.resolution)

dark_substraction = not no_dark_substraction

if dirin[-1] != '/':
    dirin = dirin + '/'

if dirout == 'default':
    dirout = dirin[:-1]+'_red/'

if not os.access(dirout,os.F_OK):
    os.makedirs(dirout, exist_ok=True)
if os.access(dirout+'proc',os.F_OK):
    os.system('rm -r '+dirout+'proc')
os.makedirs(dirout+'proc', exist_ok=True)

f_res = open(dirout+'proc/'+'results.txt','w')

if reffile == 'default':
    reffile = dirin+'reffile.txt'

if resolution != 40000 and resolution != 50000 and resolution != 60000:
        raise ValueError("Input Resolution not permited. try 40000, 50000 or 60000\n")

####### GLOBAL VARIABLES #####
force_pre_process  = False
force_bl           = False      #
force_bkg          = False
force_P            = False
force_thar_extract = False
force_thar_wavcal  = False 
force_sci_extract  = False
force_sci_proc     = True       #
force_RV           = False      #
force_stellar_pars = False
force_corr         = False

use_ref            = False
ref_traces         = '/data/echelle/duPont/red/20120630/trace.pkl'

bad_colummn        = True
have_skyF          = False

Inverse_m          = True
use_cheby          = True

MRMS              = 200
trace_degree      = 5
Marsh_alg         = 0
ext_aperture      = 6
NSigma_Marsh      = 5
NCosmic_Marsh     = 5
S_Marsh           = 0.4
N_Marsh           = 3
min_extract_col   = 50
max_extract_col   = 2000

#npar_wsol = 27 #number of parameters of wavelength solution
ncoef_x   = 5
ncoef_m   = 6
npar_wsol = (min(ncoef_x,ncoef_m) + 1) * (2*max(ncoef_x,ncoef_m) - min(ncoef_x,ncoef_m) + 2) // 2

n_last    = 55 # last echelle order as defined in the reference wavelength files
ntotal    = 64
oro0      = 36

order_dir   = os.path.join(_data_dir, 'dupont_wavcals/')

models_path = os.path.join(_pkg_dir, 'data', 'COELHO_MODELS', 'R_40000b/')

print("\n\n\tEchelle Du Pont 2.5m  PIPELINE\n")
print("\tRAW data is in ",dirin)
print("\tProducts of reduction will be in",dirout)
print('\n')

# file containing the log
log = dirout+'night.log'
biases, milkyflats, skyflats, objects, ThAr_ref, darks = dupontutils.FileClassify(dirin,log)
ThAr_ref = ThAr_ref[:2]
if dark_substraction == True and len(darks)<3:
    dark_substraction = False

f = open(log,'r')
lines = f.readlines()

print('\tThese are all the images to proccess:')
for bias in biases:
        hd = pyfits.getheader(bias)
        print('\tbias', hd['EXPTYPE'], hd['EXPTIME'], hd['UT-DATE'],hd['UT-TIME'],bias)
print('\n')
for dark in darks:
        hd = pyfits.getheader(dark)
        print('\tdark', hd['EXPTYPE'], hd['EXPTIME'], hd['UT-DATE'],hd['UT-TIME'],dark)
print('\n')
for milky in milkyflats:
        hd = pyfits.getheader(milky)
        print('\tmilky', hd['EXPTYPE'], hd['EXPTIME'], hd['UT-DATE'],hd['UT-TIME'],milky)
print('\n')
for line in lines:
        print('\t'+line[:-1])

if stst == 'last':
    if os.access(dirout+'findstar.txt',os.F_OK):
        fst = open(dirout+'findstar.txt','r')
        stst = fst.readline()
        fst.close()
    else:
        raise ValueError("There is not a previously defined standard star file!!! You have to enter one (i.e. -ofind ccd0001.fits).\n")
else:
    fst = open(dirout+'findstar.txt','w')
    fst.write(stst+'\n')
    fst.close()

if stblaze == 'same':
    print('\n\tThe pipeline will use the image that traces the orders to derive the blaze function ...')
    stblaze = stst

if ( (os.access(dirout+'Flat.fits',os.F_OK) == False) or\
   (os.access(dirout+'trace.pkl',os.F_OK) == False) or \
   (os.access(dirout+'MasterBias.fits',os.F_OK) == False) or \
   (force_pre_process) ):
    print("\tNo previous pre-processing files or found")
    pre_process = 1
else:
    print("\tPre-processing files found, going straight to extraction")
    pre_process = 0

if (pre_process == 1):
    print("\n\tGenerating Master calibration frames...")
    MasterBias, RO_bias, GA_bias = dupontutils.MedianCombine(biases, zero_bo=False, dark_bo=False, flat_bo=False)
    hdu = pyfits.PrimaryHDU( MasterBias )

    if (os.access(dirout+'MasterBias.fits',os.F_OK)):
        os.remove(dirout+'MasterBias.fits')
    hdu.writeto(dirout+'MasterBias.fits')
    print("\t\t-> Masterbias: done!")

    MDARKS = []
    dark_times = []

    if dark_substraction:
                
        for dark in darks:
                        hd = pyfits.getheader(dark)
                        if len(dark_times) == 0:
                                dark_times.append(hd['EXPTIME'])
                        else:
                                if dark_times.count(hd['EXPTIME']) == 0:
                                        dark_times.append(hd['EXPTIME'])
        dark_groups = []
        ndark_times = []        
        for time in dark_times:
            group = []
            for dark in darks:
                hd = pyfits.getheader(dark)
                if hd['EXPTIME'] == time:
                    group.append(dark)
            if len(group)>2:
                dark_groups.append(group)
                ndark_times.append(hd['EXPTIME'])

        dark_times = ndark_times
        i = 0
        while i < len(dark_times):
            DARK, RON, GAIN = dupontutils.MedianCombine(dark_groups[i], zero_bo=True, zero=dirout+'MasterBias.fits',dark_bo=False)
            hdu = pyfits.PrimaryHDU( DARK )
            hdu = GLOBALutils.update_header(hdu,'EXPTIME',dark_times[i])
            if os.access(dirout+'DARK_'+str(int(dark_times[i]))+'s.fits',os.F_OK):
                os.remove(dirout+'DARK_'+str(int(dark_times[i]))+'s.fits')
            hdu.writeto( dirout+'DARK_'+str(int(dark_times[i]))+'s.fits' )
            MDARKS.append(dirout+'DARK_'+str(int(dark_times[i]))+'s.fits')
            i+=1
    print("\t\t-> Masterdarks: done!")

    force_flat_comb = True
    if force_flat_comb or os.access(dirout+'Flat.fits',os.F_OK) == False:
        Flat, RON, GAIN = dupontutils.milk_comb(milkyflats, MDARKS, zero=dirout+'MasterBias.fits')
        hdu = pyfits.PrimaryHDU( Flat )
        hdu = GLOBALutils.update_header(hdu,'RON',RON)
        hdu = GLOBALutils.update_header(hdu,'GAIN',GAIN)
        if (os.access(dirout+'Flat_or.fits',os.F_OK)):
            os.remove(dirout+'Flat_or.fits')
        hdu.writeto(dirout+'Flat_or.fits')

        Flat = Flat/scipy.signal.medfilt(Flat,[15,15])
        hdu = pyfits.PrimaryHDU( Flat )
        hdu = GLOBALutils.update_header(hdu,'RON',RON)
        hdu = GLOBALutils.update_header(hdu,'GAIN',GAIN)

        if (os.access(dirout+'Flat.fits',os.F_OK)):
            os.remove(dirout+'Flat.fits')
        hdu.writeto(dirout+'Flat.fits')
    else:
        h = pyfits.open(dirout+'Flat.fits')
        Flat = h[0].data
        RON  = h[0].header['RON']
        GAIN = h[0].header['GAIN']
    print("\t\t-> Masterflat: done!")
        
    ftra = True
    median_filter = False

    # Find orders & traces
    print("\tTracing echelle orders...")
    if ftra or os.access(dirout+'trace.pkl',os.F_OK) == False:
        h = pyfits.open(dirin+stst)[0]
        hth = pyfits.getheader(dirin+stst)
        d = h.data
        d = dupontutils.OverscanTrim(d,hth['BIASSEC'])
        d -= MasterBias
        d /= Flat

        if bad_colummn:
            d = dupontutils.b_col(d)    

        d = d[:1420,:]  
        if use_ref:
            c_comp = pickle.load(open(ref_traces, 'rb'), encoding='latin1')['c_all']
            nord = pickle.load(open(ref_traces, 'rb'), encoding='latin1')['nord']
            lim = pickle.load(open(ref_traces, 'rb'), encoding='latin1')['lims']
            c_all,pshift = GLOBALutils.retrace( d, c_comp )
        else:
            c_all, nord = GLOBALutils.get_them(d,5,trace_degree,mode=1)
            print('\t\t'+str(nord)+' orders found...')
    else:
        trace_dict = pickle.load(open(dirout+"trace.pkl", 'rb'), encoding='latin1')
        c_all = trace_dict['c_all']
        nord = trace_dict['nord']
        GAIN = trace_dict['GA_ob']
        RON = trace_dict['RO_ob']
        
    trace_dict = {'c_all':c_all, 'nord':nord, 'GA_ob': GAIN, 'RO_ob': RON, 'DARKS':MDARKS, 'dtimes':dark_times}
    pickle.dump(trace_dict, open(dirout+"trace.pkl", 'wb'))


else:
    trace_dict = pickle.load(open(dirout+"trace.pkl", 'rb'), encoding='latin1')

    c_all = trace_dict['c_all']
    nord = trace_dict['nord']

    GAIN = trace_dict['GA_ob']
    RON = trace_dict['RO_ob']

    h = pyfits.open(dirout+'MasterBias.fits')
    MasterBias = h[0].data

    MDARKS = trace_dict['DARKS']
    dark_times = trace_dict['dtimes']

    h = pyfits.open(dirout+'Flat.fits')
    Flat = h[0].data

print('\n\tExtraction of ThAr calibration frames:')

for fsim in ThAr_ref:
    hth = pyfits.getheader(fsim)
    thmjd,mjd0 = dupontutils.mjd_fromheader(hth)
    dth = pyfits.getdata(fsim)
    dth = dupontutils.OverscanTrim(dth,hth['BIASSEC'])
    dth -= MasterBias
    dth /= Flat

    thar_fits_simple = dirout+'ThAr_'+hth['DATE-OBS']+'_'+hth['UT-TIME'][:2]+ \
                       '-'+hth['UT-TIME'][3:5]+'-'+hth['UT-TIME'][6:]+'.spec.simple.fits'

    if ( os.access(thar_fits_simple,os.F_OK) == False ) or (force_thar_extract): 
        print("\t\tNo previous extraction or extraction forced for ThAr file", fsim, "extracting...")
        close_spec = dupontutils.get_close(thmjd,hth['RA-D'],hth['DEC-D'],objects)
        temp_spec = pyfits.getdata(close_spec)
        temp_hd   = pyfits.getheader(close_spec)
        temp_spec = dupontutils.OverscanTrim(temp_spec,temp_hd['BIASSEC'])
        temp_spec -= MasterBias
        if bad_colummn:
            temp_spec = dupontutils.b_col(temp_spec)
        temp_spec = temp_spec/Flat
        c_alls, pshift = GLOBALutils.retrace( temp_spec, c_all )

        thar_Ss = GLOBALutils.simple_extraction(dth,c_alls,ext_aperture,min_extract_col,max_extract_col,npools)
        thar_Ss = thar_Ss[::-1]
        if (os.access(thar_fits_simple,os.F_OK)):
            os.remove( thar_fits_simple )
        hdu = pyfits.PrimaryHDU( thar_Ss )
        hdu.writeto( thar_fits_simple )
    else:
        print("\t\tThAr file", fsim, "all ready extracted, loading...")

print("\n\tWavelength solution of ThAr calibration spectra:")
# compute wavelength calibration files

thtimes = []
thnames = []
bad_thars = []
thRA = []
thDEC = []
ntr = 0
counter = 0
for thar in ThAr_ref:
    if ntr > 0:
        force_corr = False
    hth = pyfits.getheader(thar)
    thmjd,mjd0 = dupontutils.mjd_fromheader(hth)

    thar_fits_simple = dirout+'ThAr_'+hth['DATE-OBS']+'_'+hth['UT-TIME'][:2]+'-'+\
                       hth['UT-TIME'][3:5]+'-'+hth['UT-TIME'][6:]+'.spec.simple.fits'
    wavsol_pkl       = dirout+'ThAr' +hth['DATE-OBS']+'_'+hth['UT-TIME'][:2]+'-'+\
                       hth['UT-TIME'][3:5]+'-'+hth['UT-TIME'][6:]+'.wavsolpars.pkl'

    thar_Ss = pyfits.getdata(thar_fits_simple)

    if ( os.access(wavsol_pkl,os.F_OK) == False ) or (force_thar_wavcal):
        print(" \t\tWorking on ThAr file", thar)
        
        RON  = hth['ENOISE']
        GAIN = hth['EGAIN']
        
        lines_thar = thar_Ss[:,:]
        if os.access(dirout+'id_orders.pkl',os.F_OK) == False or force_corr:    
            maxes = 0
            or32 = 0
            for order in range(len(lines_thar)):
                ccf_max, shift = GLOBALutils.cor_thar(lines_thar[order],filename=order_dir+'order_32o.iwdat',span=50)
                if ccf_max > maxes:
                    maxes = ccf_max
                    rough_shift = shift
                    or32  =  order
            or0 = or32 - 32
                        
            if or0 >= 0:
                orwa = 0
            else:
                orwa = - or0
                or0  = 0
            #print 'n_lasts',n_last,or0 + nord
            if n_last > or0 + nord:
                n_last = or0 + nord
            if or32-32 >0:
                nn = n_last + 1
            else:
                nn = n_last + 1 + or32 -32
            #print 'NN',nn

            if os.access(dirout+'id_orders.pkl',os.F_OK):
                os.remove(dirout+'id_orders.pkl')
                        
            pdict = {'or0':or0, 'orwa':orwa, 'n_last':n_last, 'rough_shift':rough_shift,'nn':nn}
            pickle.dump(pdict, open(dirout+'id_orders.pkl', 'wb'))
            ntr += 1
                
        else:
            pdict       = pickle.load(open(dirout+'id_orders.pkl', 'rb'), encoding='latin1')
            or0         = pdict['or0']
            orwa        = pdict['orwa']
            n_last      = pdict['n_last']
            nn          = pdict['nn']
            rough_shift = pdict['rough_shift']
                
        iv_thar = 1/((lines_thar/GAIN) + (RON**2/GAIN**2))

        All_Pixel_Centers = np.array([])
        All_Wavelengths   = np.array([])
        All_Orders        = np.array([])
        All_Centroids     = np.array([])
        All_Sigmas        = np.array([])
        All_Intensities   = np.array([])
        All_Residuals     = np.array([])
        All_Sigmas        = np.array([])

        orre  = or0
        order = orwa

        OK = []
        OW = []
        
        while order < n_last:
            order_s = str(order)
            if (order < 10):
                order_s = '0'+str(order)
            thar_order_orig = lines_thar[orre,:]
            IV              = iv_thar[orre,:]
            wei             = np.sqrt( IV )
            bkg             = GLOBALutils.Lines_mBack(thar_order_orig, IV, thres_rel=3, line_w=10)
            thar_order      = thar_order_orig - bkg

            coeffs_pix2wav, coeffs_pix2sigma, pixel_centers, wavelengths, rms_ms,\
                              residuals, centroids, sigmas, intensities = GLOBALutils.Initial_Wav_Calibration( \
                              order_dir+'order_'+order_s+'o.iwdat', thar_order, order, wei, rmsmax=1000, \
                              minlines=10, FixEnds=False, Dump_Argon=False, Dump_AllLines=True, Cheby=use_cheby, \
                              rough_shift = rough_shift)

            fwhms_lns = sigmas*2.355
            inis_lns  = pixel_centers - fwhms_lns*0.5
            fins_lns  = pixel_centers + fwhms_lns*0.5                   
            inis_wvs  = GLOBALutils.Cheby_eval(coeffs_pix2wav,inis_lns,float(len(thar_order)))
            fins_wvs  = GLOBALutils.Cheby_eval(coeffs_pix2wav,fins_lns,float(len(thar_order)))
            fwhms_wvs = inis_wvs - fins_wvs
            resolution2 = wavelengths / fwhms_wvs

            if wavelengths.max() > 5500 and wavelengths.min()<5500:
                print("\t\t\tmedian Resolution of order", order, '=', np.around(np.median(resolution2)))
                        
            if (order == 32): 
                if (use_cheby):
                    Global_ZP = GLOBALutils.Cheby_eval( coeffs_pix2wav, 1023, len(thar_order) )
                else:
                    Global_ZP = np.polyval( coeffs_pix2wav, 0.0 )

            All_Pixel_Centers = np.append( All_Pixel_Centers, pixel_centers )
            All_Wavelengths   = np.append( All_Wavelengths, wavelengths )
            All_Orders        = np.append( All_Orders, np.zeros( len(pixel_centers) ) + order )
            All_Centroids     = np.append( All_Centroids, centroids)
            All_Sigmas        = np.append( All_Sigmas, sigmas)
            All_Intensities   = np.append( All_Intensities, intensities )
            All_residuals     = np.append( All_Residuals, residuals)
            All_Sigmas        = np.append( All_Sigmas,sigmas)
                        
            order += 1
            orre  += 1

        p0    = np.zeros( npar_wsol )
        p0[0] = (32+oro0) * Global_ZP

        p1, G_pix, G_ord, G_wav, II, rms_ms, G_res = \
                        GLOBALutils.Fit_Global_Wav_Solution(All_Pixel_Centers, All_Wavelengths, All_Orders, np.ones(All_Intensities.shape), p0, npix=len(lines_thar[orre,:]), Cheby=use_cheby, maxrms=150, Inv=Inverse_m, minlines=800,order0=oro0,nx=ncoef_x,nm=ncoef_m,ntotal=ntotal)

        if rms_ms/np.sqrt(float(len(G_pix))) < 20:
            pdict = {'p1':p1, 'G_pix':G_pix, 'G_ord':G_ord, 'G_wav':G_wav, 'II':II,\
                     'rms_ms':rms_ms, 'G_res':G_res, 'All_Centroids':All_Centroids,\
                     'All_Sigmas':All_Sigmas, 'or0':or0, 'orwa':orwa, 'oro0':oro0,\
                     'rough_shift':rough_shift}
            pickle.dump(pdict, open(wavsol_pkl, 'wb'))
            thtimes.append(thmjd)
            thRA.append(hth['RA-D'])
            thDEC.append(hth['DEC-D'])
            thnames.append(wavsol_pkl)
        else:
            bad_thars.append(counter)
    else:
        print("\t\tUsing previously computed wavelength solution in file",wavsol_pkl)
        pdict_temp = pickle.load(open(wavsol_pkl, 'rb'), encoding='latin1')
        if pdict_temp['rms_ms']/np.sqrt(float(len(pdict_temp['G_pix']))) < 20:
            thtimes.append(thmjd)
            thRA.append(hth['RA-D'])
            thDEC.append(hth['DEC-D'])
            thnames.append(wavsol_pkl)
        else:
            bad_thars.append(counter)
    counter += 1

bad_thars = np.array(bad_thars)
if len(bad_thars)>0:
    ThAr_ref = np.delete(ThAr_ref,bad_thars)

pdict = pickle.load(open(dirout+'id_orders.pkl', 'rb'), encoding='latin1')
or0         = pdict['or0']
orwa        = pdict['orwa']
n_last      = pdict['n_last']
nn          = pdict['nn']
rough_shift = pdict['rough_shift']

thtimes,thnames,thRA,thDEC = np.array(thtimes),np.array(thnames),np.array(thRA),np.array(thDEC)
SI = np.argsort(thtimes)
thtimes,thnames,thRA,thDEC = thtimes[SI],thnames[SI],thRA[SI],thDEC[SI]

"""
# get the reference P matrix
bkg_obj_fits = dirout + 'Bkg_ref.fits'
P_fits       = dirout + 'P_ref.fits'

dat = pyfits.getdata(dirin+stst)
hd  = pyfits.getheader(dirin+stst)

RON, GAIN = hd['ENOISE'], hd['EGAIN']
dat = utils.OverscanTrim(dat,hd['BIASSEC'])
dat -= MasterBias

if dark_substraction and hd['EXPTIME'] > 600:
        dat -= utils.get_dark(MDARKS, hd['EXPTIME'])
if bad_colummn:
        dat = utils.b_col(dat)
dat = dat/Flat
                
print('Recentering traces...')
c_alls,pshift = GLOBALutils.retrace( dat, c_all )

Centers = np.zeros((len(c_alls),dat.shape[1]))
for i in range(nord):
        Centers[i,:]=np.polyval(c_alls[i,:],np.arange(len(Centers[i,:])))
print('Scatter Light Determination...')
if np.median(dat[Centers[0,1024]:Centers[-1,1024],1024])> 0:
        if ( os.access(bkg_obj_fits,os.F_OK) == False or force_bkg):
                #bkg = bkg_simple.get_scat(dat,Centers)
                bkg = GLOBALutils.get_scat(dat,Centers,span=6)
                if (os.access(bkg_obj_fits,os.F_OK)):
                        os.remove( bkg_obj_fits )
                hdu = pyfits.PrimaryHDU( bkg )
                hdu.writeto( bkg_obj_fits )
        else:
                bkg = pyfits.getdata(bkg_obj_fits)
        dat -= bkg

print('P matrix determination...')
force_P = False
if os.access(P_fits,os.F_OK) == False or force_P:
        P  = GLOBALutils.obtain_P(dat,c_alls,ext_aperture,RON,GAIN,NSigma_Marsh, S_Marsh,N_Marsh, Marsh_alg,0,2047,npools)
        if (os.access(P_fits,os.F_OK)):
                os.remove( P_fits )
        hdu = pyfits.PrimaryHDU( P )
        hdu.writeto( P_fits )
else:
        P = pyfits.getdata(P_fits)
"""

print('\n\tProcessing of science images:')

sthd   = pyfits.getheader(dirin+stblaze)
stname = sthd['OBJECT']
new_list         = []
new_list_obnames = []
new_list_texp    = []
for i in range(len(objects)):
    fsim = objects[i]
    hd = pyfits.getheader(objects[i])
    obname = hd['OBJECT']
    texp = hd['EXPTIME']
    if object2do == 'all':
        new_list.append(fsim)
        new_list_obnames.append( obname )
        new_list_texp.append( texp )
    else:
        if (obname == object2do) or obname == stname:
            new_list.append(fsim)
            new_list_obnames.append( obname )
            new_list_texp.append( texp )
objects=new_list
objects = objects[:5]

for obj in objects:

        print('\n')
        print("\t--> Working on image: ", obj)

        hd = pyfits.getheader(obj)
        nombre    = hd['OBJECT']
        RON, GAIN = hd['ENOISE'], hd['EGAIN']

        print("\t\tObject name:",nombre)
        
        nama = nombre + '_' + hd['DATE-OBS'] + '_' + hd['UT-TIME'][:2] +\
               '-' + hd['UT-TIME'][3:5]+'-' + hd['UT-TIME'][6:]

        obj_fits        = dirout + nama + '.spec.fits.S'
        obj_fits_simple = dirout + nama + '.spec.simple.fits.S'
        bkg_obj_fits    = dirout + 'Bkg_' + nama + '.fits'      
        P_fits          = dirout + 'P_' + nama + '.fits'

        only_simple   = False
        if stname == nombre:
            stpath = obj_fits_simple

        cond2 = False
        if only_simple:
                if ( os.access(obj_fits_simple,os.F_OK) == False ) or (force_sci_extract) :
                        cond2 = True
        else:
                if ( os.access(obj_fits,os.F_OK) == False )  or ( os.access(obj_fits_simple,os.F_OK) == False ) or (force_sci_extract) or ( os.access(P_fits,os.F_OK) == False ):
                        cond2 = True

        if cond2:

                print("\t\tNo previous extraction or extraction forced for science file", obj, "extracting...")

                dat = pyfits.getdata(obj)
                hdt = pyfits.getheader(obj)
                dat = dupontutils.OverscanTrim(dat,hdt['BIASSEC'])
                dat -= MasterBias
                if dark_substraction and hd['EXPTIME'] >= 899:
                    dat -= dupontutils.get_dark(MDARKS, hd['EXPTIME'])

                if bad_colummn:
                    dat = dupontutils.b_col(dat)
                dat = dat/Flat
                
                c_alls,pshift = GLOBALutils.retrace( dat, c_all )

                Centers = np.zeros((len(c_alls),dat.shape[1]))
                for i in range(nord):
                        Centers[i,:]=np.polyval(c_alls[i,:],np.arange(len(Centers[i,:])))

                #print 'Scatter Light Determination...'

                #print 'SN',np.median(dat[Centers[0,1024]:Centers[-1,1024],1024])
                if np.median(dat[Centers[0,1024]:Centers[-1,1024],1024])> 0:
                        if ( os.access(bkg_obj_fits,os.F_OK) == False or force_bkg):
                                bkg = GLOBALutils.get_scat(dat,Centers,span=6)
                                if (os.access(bkg_obj_fits,os.F_OK)):
                                        os.remove( bkg_obj_fits )
                                hdu = pyfits.PrimaryHDU( bkg )
                                hdu.writeto( bkg_obj_fits )
                        else:
                                bkg = pyfits.getdata(bkg_obj_fits)
                        dat -= bkg
                else:
                        NCosmic_Marsh = 6

                #print 'P matrix determination...'
                use_ref_P = False
                if use_ref_P:
                        P = pyfits.getdata(dirout + 'P_ref.fits')
                        Pn = P.copy()
                        for i in range(P.shape[0]):
                                j = i - pshift
                                if j < 0 or j>=P.shape[0]:
                                        Pn[i] = 0.
                                else:
                                        Pn[i] = P[j]
                        P = Pn.copy()

                else:
                        if os.access(P_fits,os.F_OK) == False or force_P:
                                P = GLOBALutils.obtain_P(dat,c_alls,ext_aperture,RON,\
                                    GAIN,NSigma_Marsh, S_Marsh,N_Marsh, Marsh_alg,0,2047,npools)
                                if (os.access(P_fits,os.F_OK)):
                                        os.remove( P_fits )
                                hdu = pyfits.PrimaryHDU( P )
                                hdu.writeto( P_fits )
                        else:
                                P = pyfits.getdata(P_fits)

                if only_simple==False:
                        obj_S  = GLOBALutils.optimal_extraction(dat,P,c_alls,ext_aperture,RON,\
                                 GAIN,S_Marsh,NCosmic_Marsh,50,2000,npools)
                        obj_S  = obj_S[::-1]
                        if (os.access(obj_fits,os.F_OK)):
                                os.remove( obj_fits )
                        hdu = pyfits.PrimaryHDU( obj_S )
                        hdu.writeto( obj_fits )

                obj_Ss = GLOBALutils.simple_extraction(dat,c_alls,ext_aperture,min_extract_col,max_extract_col,npools)
                obj_Ss = obj_Ss[::-1]
                
                if (os.access(obj_fits_simple,os.F_OK)):
                        os.remove( obj_fits_simple )
                
                hdu = pyfits.PrimaryHDU( obj_Ss )
                hdu.writeto( obj_fits_simple )

###################################################################################################################
############################################### get blaze function ################################################
###################################################################################################################
print('\n\tComputing the blaze function using the standard star...')

if os.access(dirout+'blaze_f.fits',os.F_OK) == False or force_bl:               
    index = -1
    hd = pyfits.getheader(dirin+stblaze)
    nombre  = hd['OBJECT']
    exptime = hd['EXPTIME']     
    RA      = hd['RA-D']
    DEC     = hd['DEC-D']
    RON     = hd['ENOISE']
    GAIN    = hd['EGAIN']
    scmjd,scmjd0 = dupontutils.mjd_fromheader(hd)
    altitude     = hd['SITEALT']
    latitude     = hd['SITELAT']
    longitude    = hd['SITELONG']
    epoch        = hd['EPOCH']

    iers          = GLOBALutils.JPLiers( baryc_dir, scmjd-999.0, scmjd+999.0 )
    obsradius, R0 = GLOBALutils.JPLR0( latitude, altitude)
    obpos         = GLOBALutils.obspos( longitude, obsradius, R0 )

    jplephem.set_ephemeris_dir( baryc_dir , ephemeris )
    jplephem.set_observer_coordinates( obpos[0], obpos[1], obpos[2] )

    res           = jplephem.doppler_fraction(RA/15.0, DEC, int(scmjd), scmjd%1, 1, 0.0)
    lbary_ltopo   = 1.0 + res['frac'][0]
    bcvel_baryc   = ( lbary_ltopo - 1.0 ) * 2.99792458E5

    res  = jplephem.pulse_delay(RA/15.0, DEC, int(scmjd), scmjd%1, 1, 0.0)   
    mbjd = scmjd + res['delay'][0] / (3600.0 * 24.0)

    gobs      = ephem.Observer()
    gobs.name = 'DUPONT'
    gobs.lat  = rad(latitude)  # lat/long in decimal degrees  
    gobs.long = rad(longitude)
    gobs.date = hd['UT-DATE'].replace('-','/') + ' ' + hd['UT-TIME']

    mephem = ephem.Moon()
    mephem.compute(gobs)
    Mcoo = jplephem.object_track("Moon", int(scmjd), float(scmjd%1), 1, 0.0)
    Mp   = jplephem.barycentric_object_track("Moon", int(scmjd), float(scmjd%1), 1, 0.0)
    Sp   = jplephem.barycentric_object_track("Sun", int(scmjd), float(scmjd%1), 1, 0.0)
    res  = jplephem.object_doppler("Moon", int(scmjd), scmjd%1, 1, 0.0)
    lunation,moon_state,moonsep2,moonvel = GLOBALutils.get_lunar_props(ephem,gobs,Mcoo,Mp,Sp,res,RA,DEC)
    refvel = bcvel_baryc + moonvel
    print('\t\tBarycentric velocity:',refvel)

    i=0
    mind = 1000.0
    for t in thtimes:
        if np.abs(t-scmjd)<mind:
            mind  = np.abs(t-scmjd)
            index = i
        i+=1

    obj_S = pyfits.getdata(stpath)

    wavsol_pkl1 = thnames[index]
    pdict1      = pickle.load(open(wavsol_pkl1, 'rb'), encoding='latin1')
    global1     = pdict1['p1']
    or01        = pdict1['or0']
    orwa1       = pdict1['orwa']
    oro0        = pdict1['oro0']

    final  = np.zeros( [3, n_last-orwa1, np.shape(obj_S)[1]] )
    equis  = np.arange( np.shape(obj_S)[1] ) 
    order  = orwa1
    orre   = or01
    final2 = np.zeros( [nn, np.shape(obj_S)[1]] )
    while order < n_last:
        m = order + oro0
        chebs = GLOBALutils.Calculate_chebs(equis, m, order0=oro0, ntotal=ntotal, npix=obj_S.shape[1], Inverse=Inverse_m, nx=ncoef_x,nm=ncoef_m )
        WavSol = lbary_ltopo*(1.0/m) * GLOBALutils.Joint_Polynomial_Cheby(global1,chebs,ncoef_x,ncoef_m)
        final[0,orre,:] = GLOBALutils.ToVacuum(WavSol[::-1])
        final[1,orre,:] = obj_S[orre,:][::-1]
        order += 1
        orre += 1

    final[2,:,:] = dupontutils.get_blaze(final[0,:,:],final[1,:,:])
                
    blaze = np.zeros((2,final.shape[1],final.shape[2]))
    blaze[0] = final[0]
    blaze[1] = dupontutils.get_blaze(final[0,:,:],final[1,:,:])
        
    if (os.access(dirout+'blaze_f.fits',os.F_OK)):
        os.remove( dirout+'blaze_f.fits' )
    hdu = pyfits.PrimaryHDU( blaze )
    hdu.writeto(dirout+'blaze_f.fits'  )
else:
    blaze = pyfits.getdata(dirout+'blaze_f.fits' )


for i in range(blaze.shape[1]):
    blaze[1,i]/=blaze[1,i].max()

#################################################################################################################
############################################  Final output ######################################################
#################################################################################################################

print("\n\tBuilding the final output spectra...")

for obj in objects:
    hd     = pyfits.getheader(obj)
    nombre = hd['OBJECT']
    nama   = nombre+'_'+hd['DATE-OBS']+'_'+hd['UT-TIME'][:2]+'-'+hd['UT-TIME'][3:5]+'-'+hd['UT-TIME'][6:]
    nf     = nama+'_final.fits'
    nfS    = nama+'_finalS.fits'
    print("\n\t\t-->Building", nf)
    if (os.access(dirout+'proc/'+nf,os.F_OK)) == False or force_sci_proc:
                        index1,index2 = -1,-1
                        
                        # Get observing info from header
                        hd = pyfits.getheader(obj)
                        nombre    = hd['OBJECT']
                        exptime   = hd['EXPTIME']
                        RA        = hd['RA-D']
                        DEC       = hd['DEC-D']
                        RON       = hd['ENOISE']
                        GAIN      = hd['EGAIN']
                        altitude  = hd['SITEALT']
                        latitude  = hd['SITELAT']
                        longitude = hd['SITELONG']
                        epoch     = hd['EPOCH']

                        scmjd,scmjd0 = dupontutils.mjd_fromheader(hd)
                        ra2,dec2 = GLOBALutils.getcoords(obname,scmjd,filen=reffile)
                        if ra2 !=0 and dec2 != 0:
                                RA = ra2
                                DEC = dec2
                        else:
                                print('\t\tUsing the coordinates found in the image header.')

                        # set info for compute the baricentric correction
                        iers          = GLOBALutils.JPLiers( baryc_dir, scmjd-999.0, scmjd+999.0 )
                        obsradius, R0 = GLOBALutils.JPLR0( latitude, altitude)
                        obpos         = GLOBALutils.obspos( longitude, obsradius, R0 )
                        jplephem.set_ephemeris_dir( baryc_dir , ephemeris )
                        jplephem.set_observer_coordinates( obpos[0], obpos[1], obpos[2] )
                        res         = jplephem.doppler_fraction(RA/15.0, DEC, int(scmjd), scmjd%1, 1, 0.0)
                        lbary_ltopo = 1.0 + res['frac'][0]
                        bcvel_baryc = ( lbary_ltopo - 1.0 ) * 2.99792458E5      #This in the barycentric velocity
                        res         = jplephem.pulse_delay(RA/15.0, DEC, int(scmjd), scmjd%1, 1, 0.0)
                        scmbjd      = scmjd + res['delay'][0] / (3600.0 * 24.0) #This is the modified barycentric julian day of the observation

                        # set observatory info to retrive info about the moon
                        gobs = ephem.Observer()
                        gobs.name = 'DUPONT'
                        gobs.lat  = rad(latitude)
                        gobs.long = rad(longitude)
                        #gobs.date = hd['UT-DATE'] + ' ' + hd['UT-TIME'].replace(':','_')
                        gobs.date = hd['UT-DATE'].replace('-','/') + ' ' + hd['UT-TIME']
                
                        mephem = ephem.Moon()
                        mephem.compute(gobs)
                        Mcoo = jplephem.object_track("Moon", int(scmjd), float(scmjd%1), 1, 0.0)
                        Mp   = jplephem.barycentric_object_track("Moon", int(scmjd), float(scmjd%1), 1, 0.0)
                        Sp   = jplephem.barycentric_object_track("Sun", int(scmjd), float(scmjd%1), 1, 0.0)
                        res      = jplephem.object_doppler("Moon", int(scmjd), scmjd%1, 1, 0.0)
                        lunation,moon_state,moonsep2,moonvel = GLOBALutils.get_lunar_props(ephem,gobs,Mcoo,Mp,Sp,res,RA,DEC)
                        refvel = bcvel_baryc + moonvel  #This is the velocity of the spectrum of the moon with the applied barycentric correction in the direction of the target. 
                        
                        print('\t\t\tBarycentric velocity:',refvel)

                        # Set the ThAr lamps for applying the wavelength solution
                        if scmjd < thtimes[0]:
                                "\t\t\tProblem with ThAr and science times"
                                index1 = 0
                                index2 = 0
                        elif scmjd > thtimes[-1]:
                                "\t\t\tProblem with ThAr and science times"
                                index1 = -1
                                index2 = -1
                        else:
                                "\t\t\tThAr images taken before and after science image found..."
                                for i in range(len(thtimes)-1):
                                        if scmjd >= thtimes[i] and scmjd < thtimes[i+1]:
                                                index1 = i
                                                index2 = i+1
                                                break
                        
                        if np.abs(RA-thRA[index1]) > 0.005 or np.abs(DEC-thDEC[index1]) > 0.005:
                                index1 = index2
                        if np.abs(RA-thRA[index2]) > 0.005 or np.abs(DEC-thDEC[index2]) > 0.005:
                                index2 = index1
                
                        obj_fits        = dirout+nama+'.spec.fits.S'
                        obj_fits_simple = dirout+nama+'.spec.simple.fits.S'
                        obj_S           = pyfits.getdata(obj_fits)
                        obj_Ss          = pyfits.getdata(obj_fits_simple)
        
                        wavsol_pkl1 = thnames[index1]
                        pdict1      = pickle.load(open(wavsol_pkl1, 'rb'), encoding='latin1')
                        global1     = pdict1['p1']
                        or01        = pdict1['or0']
                        orwa1       = pdict1['orwa']
                        oro0        = pdict1['oro0']
                        wavsol_pkl2 = thnames[index2]
                        pdict2      = pickle.load(open(wavsol_pkl2, 'rb'), encoding='latin1')
                        global2     = pdict2['p1']

                        final  = np.zeros( [11, nn, np.shape(obj_S)[2]] )
                        finalS = np.zeros( [2, nn, np.shape(obj_Ss)[1]] )
                        equis  = np.arange( np.shape(obj_S)[2] ) 
                        order  = orwa1
                        orre   = or01

                        while order < n_last:
                                
                                m = order + oro0
                                chebs = GLOBALutils.Calculate_chebs(equis, m, order0=oro0, ntotal=ntotal, npix=obj_S.shape[2], Inverse=Inverse_m,nx=ncoef_x,nm=ncoef_m)
                                WavSol  = lbary_ltopo*(1.0/m) * GLOBALutils.Joint_Polynomial_Cheby(global1,chebs,ncoef_x,ncoef_m)
                                WavSol2 = lbary_ltopo*(1.0/m) * GLOBALutils.Joint_Polynomial_Cheby(global2,chebs,ncoef_x,ncoef_m)
                                
                                final[0,orre,:] = GLOBALutils.ToVacuum(0.5*(WavSol[::-1]+WavSol2[::-1]))
                                final[1,orre,:] = obj_S[orre,1,:][::-1]
                                final[2,orre,:] = obj_S[orre,2,:][::-1]
                                tck = scipy.interpolate.splrep(blaze[0,orre,:],blaze[1,orre,:],k=3,s=0)
                                blin = scipy.interpolate.splev(final[0,orre,:],tck,der = 0)
                                final[3,orre,:] = final[1,orre,:]/blin
                                final[4,orre,:] = final[2,orre,:]*(blin**2)
                                
                                X = final[0,orre,:]
                                Z = final[3,orre,:]
                                Z[:150]  = 0.0
                                Z[-150:] = 0.0

                                I    = np.isnan(Z)
                                Z[I] = 0.0
                                I    = np.where(Z!=0)[0]
                                if len(I)>0:                    
                                        cp    = continuum.NORM_single(X,Z,orden=1)
                                        ratio = np.polyval(cp,X)
                                else:
                                        ratio = np.ones(len(X))

                                ratio = np.polyval(cp,X)
                                final[5,orre,:] = final[3,orre,:]/ratio
                                Inan = np.where( np.isnan(final[1,orre,:]) == True )[0]
                                final[5,orre,Inan] = 1.
                                final[6,orre,:] = final[4,orre,:]*(ratio**2)
                                final[7,orre,:] = ratio
                                final[8,orre,:] = ratio*blin / np.sqrt( ratio *blin / GAIN + (RON/GAIN)**2 )
                                
                                spl        = scipy.interpolate.splrep(np.arange(len(final[0,orre,:])),final[0,orre,:] ,k=3)
                                dlambda_dx = scipy.interpolate.splev(np.arange(len(final[0,orre,:])), spl, der=1)
                                NN         = np.average(dlambda_dx)
                                dlambda_dx /= NN
                                
                                LL               = np.where( final[5,orre,:] != 0 )[0]
                                medflx           = scipy.signal.medfilt( final[5,orre,LL], 3 )
                                fres             = final[5,orre,LL] - medflx
                                fdev             = np.sqrt(np.var(fres))
                                II               = np.where(final[5,orre,:] > 1. + 5*fdev)[0]
                                final[5,orre,II] = 1.

                                final[9,orre,:]  = final[5,orre,:] * (dlambda_dx ** 1)
                                final[10,orre,:] = final[6,orre,:] / (dlambda_dx ** 2)

                                finalS[0,orre,:] = WavSol[::-1]
                                finalS[1,orre,:] = obj_Ss[orre,:][::-1]


                                order += 1
                                orre += 1

                        hdu = pyfits.PrimaryHDU( final )
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH MJD',scmjd)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH MBJD',scmbjd)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH SHUTTER START DATE',hd['DATE-OBS'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH SHUTTER START UT',hd['UT-TIME'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH TEXP (S)',hd['EXPTIME'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH BARYCENTRIC CORRECTION (KM/S)',bcvel_baryc,'[km/s]')
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH (LAMBDA_BARY / LAMBDA_TOPO)',lbary_ltopo)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH TARGET NAME',nombre)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH RA',hd['RA'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH DEC',hd['DEC'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH RA-D',hd['RA-D'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH DEC-D',hd['DEC-D'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH RA BARY',RA)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH DEC BARY',DEC)                    
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH EQUINOX',hd['EQUINOX'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH OBS LATITUDE',hd['SITELAT'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH OBS LONGITUDE',hd['SITELONG'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH OBS ALTITUDE',hd['SITEALT'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH TARG AIRMASS',hd['AIRMASS'])
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH MOON_VEL',refvel,'[km/s]')
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH MOONST',moon_state)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH LUNATION',lunation)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH MOONSEP',moonsep2)
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH MOONALT',float(mephem.alt))
                        hdu = GLOBALutils.update_header(hdu,'HIERARCH SMOONALT',str(mephem.alt))
                
                        if (os.access(dirout+'proc/'+nf,os.F_OK)):
                                        os.remove( dirout+'proc/'+nf )
                        hdu.writeto( dirout+'proc/'+nf )
                
#####################################################################################################################
############################################### Spectral and RV analysis ############################################
#####################################################################################################################

# Does any image have a special requirement for dealing with the moonlight?
if os.access(dirin + 'moon_corr.txt', os.F_OK):
    fmoon = open(dirin + 'moon_corr.txt','r')
    moon_lns = fmoon.readlines()
    spec_moon = []
    use_moon = []
    for line in moon_lns:
        spec_moon.append(line.split()[0])
        if line.split()[1] == '0':
            use_moon.append(False)
        else:
            use_moon.append(True)
else:
    spec_moon = []
    use_moon = []

spec_moon = np.array(spec_moon)
use_moon  = np.array(use_moon)

print('Starting with the Post-processing of the spectra...')

if not JustExtract:

    for obj in objects:
        hd     = pyfits.getheader(obj)
        nombre = hd['OBJECT']
        nama   = dirout+'proc/' + nombre+'_'+hd['DATE-OBS']+'_'+hd['UT-TIME'][:2]+'-'+hd['UT-TIME'][3:5]+'-'+hd['UT-TIME'][6:]
        fit    = nama+'_final.fits'
        print('\n\t\tWorking on spectrum:', fit)
        know_moon = False
        if fsim.split('/')[-1] in spec_moon:
            I = np.where(obj.split('/')[-1] == spec_moon)[0]
            know_moon = True
            here_moon = use_moon[I]

        hd   = pyfits.getheader(fit)
        spec = pyfits.getdata(fit)
        hdu  = pyfits.open(fit,'update')

        obname      = hd['TARGET NAME']
        refvel      = hd['MOON_VEL']
        lunation    = hd['LUNATION']
        moon_state  = hd['MOONST']
        moonsep     = hd['MOONSEP']
        moon_alt    = hd['MOONALT']
        smoon_alt   = hd['SMOONALT']
        lbary_ltopo = hd['(LAMBDA_BARY / LAMBDA_TOPO)']
        mjd         = hd['MJD']
        TEXP        = hd['TEXP (S)']
        mbjd        = hd['MBJD']

        for i in range(spec.shape[1]):
            if spec[0,i,500] < 5200:
                tuc = i
                break
        SNR_5130 = np.median(spec[8,tuc][800:1200] )

        if DoClass:
            print('\t\tSpectral Analysis:')
            # spectral analysis
            # First, query SIMBAD with the object name
            query_success = False
            query_success,sp_type_query = GLOBALutils.simbad_query_obname(obname)
            # Now, query SIMBAD by coordinates if above not successful
            if (not query_success):
                query_success,sp_type_query = GLOBALutils.simbad_query_coords('12:00:00','00:00:00')
            print("\t\t\tSpectral type returned by SIMBAD query:",sp_type_query)

            hdu[0] = GLOBALutils.update_header(hdu[0],'HIERARCH SIMBAD SPTYP', sp_type_query)

            pars_file = dirout + nombre+'_'+hd['HIERARCH SHUTTER START DATE']+'_' \
                                + hd['HIERARCH SHUTTER START UT'][:2]+'-'+hd['HIERARCH SHUTTER START UT'][3:5]+ \
                                '-' + hd['HIERARCH SHUTTER START UT'][6:] + '_stellar_pars.txt'

            if os.access(pars_file,os.F_OK) == False or force_stellar_pars:
                print("\t\t\tEstimating atmospheric parameters:")
                spec2 = spec.copy()
                if resolution > 45000:
                    Rx = np.around(1./np.sqrt(1./40000.**2 - 1./resolution**2))
                    for i in range(spec.shape[1]):
                        IJ = np.where(spec[5,i]!=0.)[0]
                        spec2[5,i,IJ] = GLOBALutils.convolve(spec[0,i,IJ],spec[5,i,IJ],Rx)
                T_eff, logg, Z, vsini, vel0, ccf = correlation.CCF(spec2,model_path=models_path,npools=npools)
                line = "%6d %4.1f %4.1f %8.1f %8.1f\n" % (T_eff,logg, Z, vsini, vel0)
                f = open(pars_file,'w')
                f.write(line)
                f.close()
            else:
                print("\t\t\tAtmospheric parameters loaded from file:")
                T_eff, logg, Z, vsini, vel0 = np.loadtxt(pars_file,unpack=True)

            print("\t\t\t\tT_eff=",T_eff,"np.log(g)=",logg,"Z=",Z,"vsin(i)=",vsini,"vel0",vel0)

        else:
            T_eff, logg, Z, vsini, vel0 = -999,-999,-999,-999,-999

        # store the parameters measured for this epoch
        T_eff_epoch = T_eff
        logg_epoch  = logg
        Z_epoch     = Z
        vsini_epoch = vsini
        vel0_epoch  = vel0
        
        hdu[0] = GLOBALutils.update_header(hdu[0],'HIERARCH VEL0', vel0)
        hdu[0] = GLOBALutils.update_header(hdu[0],'HIERARCH TEFF', float(T_eff))
        hdu[0] = GLOBALutils.update_header(hdu[0],'HIERARCH LOGG', float(logg))
        hdu[0] = GLOBALutils.update_header(hdu[0],'HIERARCH Z', Z)
        hdu[0] = GLOBALutils.update_header(hdu[0],'HIERARCH VSINI', vsini)

        print("\t\tRadial Velocity analysis:")
        # assign mask
        sp_type, mask = GLOBALutils.get_mask_reffile(obname,reffile=reffile,base=_xc_masks_dir)
        print("\t\t\tWill use",sp_type,"mask for CCF.")

        first_o = 0
        for i in range(spec.shape[1]):
            if spec[0,i,0]<6500:
                first_o = i
                break

        last_o = spec.shape[1]
        for i in range(spec.shape[1]):
            if spec[0,i,0]<4300:
                last_o = i
                break

        spec1 = spec[:,first_o:last_o+1,:]
        
        # Read in mask
        ml, mh, weight = np.loadtxt(mask,unpack=True)
        ml_v = GLOBALutils.ToVacuum( ml )
        mh_v = GLOBALutils.ToVacuum( mh )

        # make mask larger accounting for factor ~3 lower res in DUPONT w/r to HARPS
        av_m = 0.5*( ml_v + mh_v )
        ml_v -= 1.5*(av_m - ml_v)
        mh_v += 1.5*(mh_v - av_m)
        mask_hw_kms = (GLOBALutils.Constants.c/1e3) * 0.5*(mh_v - ml_v) / av_m

        disp = GLOBALutils.get_disp(obname, reffile=reffile)

        if disp == 0:
            known_sigma = False
            if vsini != -999 and vsini != 0.:
                disp = vsini
            else:
                disp = 3.
        else:
            known_sigma = True

        mask_hw_wide = av_m * disp / (GLOBALutils.Constants.c/1.0e3)
        ml_v = av_m - mask_hw_wide
        mh_v = av_m + mask_hw_wide
                
        print('\t\t\tComputing the CCF...')
        cond = True
        while (cond):
            #first rough correlation to find the minimum
            vels, xc_full, sn, nlines_ccf, W_ccf = GLOBALutils.XCor(spec1, ml_v, mh_v,\
                                            weight, 0, lbary_ltopo, vel_width=300, vel_step=3, start_order=0,\
                                            spec_order=9, iv_order=10, sn_order=8,max_vel_rough=300)
            xc_av = GLOBALutils.Average_CCF(xc_full, sn, sn_min=3, Simple=True, start_order=0, W=W_ccf)
            #Normalize the continuum of the CCF robustly with R     
            yy = scipy.signal.medfilt(xc_av,11)
            pred = lowess(yy, vels,frac=0.4,it=10,return_sorted=False)
            tck1 = scipy.interpolate.splrep(vels,pred,k=1)
            xc_av_orig = xc_av.copy()
            xc_av /= pred
            pred_rough = pred.copy()

            vel0_xc = vels[ np.argmin( xc_av ) ] 

            rvels, rxc_av, rpred, rxc_av_orig, rvel0_xc = vels.copy(), \
                                xc_av.copy(), pred.copy(), xc_av_orig.copy(), vel0_xc
            xc_av_rough = xc_av
            vels_rough  = vels
                        
            vel_width = np.maximum( 20.0, 6*disp )
            vels, xc_full, sn, nlines_ccf, W_ccf =\
                                        GLOBALutils.XCor(spec1, ml_v, mh_v, weight, vel0_xc, lbary_ltopo,\
                                        start_order=0, vel_width=vel_width, vel_step=0.1, spec_order=9, \
                                        iv_order=10, sn_order=8,max_vel_rough=300)
                        
            xc_av = GLOBALutils.Average_CCF(xc_full, sn, sn_min=3,\
                                        Simple=True, W=W_ccf, start_order=0)

            pred = scipy.interpolate.splev(vels,tck1)
            xc_av /= pred

            if sp_type == 'M5':
                moon_sig = 2.5
            elif sp_type == 'K5':
                moon_sig = 3.3
            else:
                moon_sig = 4.5

            p1,XCmodel,p1gau,XCmodelgau,Ls2 = \
                                                GLOBALutils.XC_Final_Fit( vels, xc_av , sigma_res=4,\
                                                horder=8, moonv = refvel, moons = moon_sig, moon = False)

            moonmatters = False
            if (know_moon and here_moon):
                moonmatters = True
                ismoon = True
                confused = False
                p1_m,XCmodel_m,p1gau_m,XCmodelgau_m,Ls2_m = GLOBALutils.XC_Final_Fit( vels, xc_av , sigma_res = 4, horder=8, moonv = refvel, moons = moon_sig, moon = True)
                moon_flag = 1
            else:
                confused = False
                ismoon = False
                p1_m,XCmodel_m,p1gau_m,XCmodelgau_m,Ls2_m = p1,XCmodel,p1gau,XCmodelgau,Ls2
                moon_flag = 0

            bspan = GLOBALutils.calc_bss(vels,xc_av)
            SP = bspan[0]

            if (not known_sigma):
                disp = np.floor(p1gau[2])
                if (disp < 3.0): 
                    disp = 3.0
                mask_hw_wide = av_m * disp / (GLOBALutils.Constants.c/1.0e3)
                ml_v = av_m - mask_hw_wide
                mh_v = av_m + mask_hw_wide            
                known_sigma = True
            else:
                cond = False

        xc_dict = {'vels':vels,'xc_av':xc_av,'XCmodelgau':XCmodelgau,'Ls2':Ls2,'refvel':refvel,\
                       'rvels':rvels,'rxc_av':rxc_av,'rpred':rpred,'rxc_av_orig':rxc_av_orig,\
                       'rvel0_xc':rvel0_xc,'xc_full':xc_full, 'p1':p1, 'sn':sn, 'p1gau':p1gau,\
                       'p1_m':p1_m,'XCmodel_m':XCmodel_m,'p1gau_m':p1gau_m,'Ls2_m':Ls2_m,\
                       'XCmodelgau_m':XCmodelgau_m}

        moon_dict = {'moonmatters':moonmatters,'moon_state':moon_state,'moonsep':moonsep,\
                         'lunation':lunation,'mephem':mephem,'texp':TEXP}

        pkl_xc = fit[:-4] + obname + '_XC_' + sp_type + '.pkl'
        pickle.dump(xc_dict, open(pkl_xc, 'wb'))
        ccf_pdf = dirout + 'proc/' + fit.split('/')[-1][:-4] + obname + '_XCs_' + sp_type + '.pdf'
        if not avoid_plot:
            GLOBALutils.plot_CCF(xc_dict,moon_dict,path=ccf_pdf)

        airmass  = hdu[0].header['TARG AIRMASS']
        seeing   = -999

        if sp_type == 'G2':
            if T_eff < 6000:
                D = 0.24416
                C = 0.00181
            else:
                D = 0.33491
                C = 0.00113
        elif  sp_type == 'K5':  
            D = 0.20695
            C = 0.00321
        else:   
            D = 0.20695
            C = 0.00321

        RV     = np.around(p1gau_m[1],4)  
        BS     = np.around(SP,4)  
        RVerr2 = 0.400
        BSerr  = np.around(D / float(np.round(SNR_5130)) + C,4)

        print('\t\t\tRV = '+str(RV)+' +- '+str(RVerr2))
        print('\t\t\tBS = '+str(BS)+' +- '+str(BSerr))

        bjd_out = 2400000.5 + mbjd
        T_eff_err = 100
        logg_err = 0.5
        Z_err = 0.5
        vsini_err = 2
        XC_min = np.abs(np.around(np.min(XCmodel),2))

        SNR_5130 = np.around(SNR_5130)
        SNR_5130_R = np.around(SNR_5130*np.sqrt(2.9))

        disp_epoch = np.around(p1gau_m[2],1)
        hdu[0] = GLOBALutils.update_header(hdu[0],'RV', RV)
        hdu[0] = GLOBALutils.update_header(hdu[0],'RV_E', RVerr2)
        hdu[0] = GLOBALutils.update_header(hdu[0],'BS', BS)
        hdu[0] = GLOBALutils.update_header(hdu[0],'BS_E', BSerr)
        hdu[0] = GLOBALutils.update_header(hdu[0],'DISP', disp_epoch)
        hdu[0] = GLOBALutils.update_header(hdu[0],'SNR', SNR_5130)
        hdu[0] = GLOBALutils.update_header(hdu[0],'SNR_R', SNR_5130_R)
        hdu[0] = GLOBALutils.update_header(hdu[0],'INST', 'DUPONT')
        hdu[0] = GLOBALutils.update_header(hdu[0],'RESOL', resolution)
        hdu[0] = GLOBALutils.update_header(hdu[0],'PIPELINE', 'CERES')
        hdu[0] = GLOBALutils.update_header(hdu[0],'XC_MIN', XC_min)
        hdu[0] = GLOBALutils.update_header(hdu[0],'BJD_OUT', bjd_out)

        line_out = "%-15s %18.8f %9.4f %7.4f %9.3f %5.3f   dupont   ceres   %8d %6d %5.2f %5.2f %5.1f %4.2f %5.2f %6.1f %4d %s\n"%\
                      (obname, bjd_out, RV, RVerr2, BS, BSerr, resolution, T_eff_epoch, logg_epoch,\
                       Z_epoch, vsini_epoch, XC_min, disp_epoch, TEXP, SNR_5130_R, ccf_pdf)
        f_res.write(line_out)
        hdu.close()
f_res.close()



import numpy

#inp = 'B_order_00.dat'
ii = 10
while ii < 44:
        inp = 'B_order_'+str(ii)+'.dat'
        print(inp)
        f = open(inp,'r')
        lines = f.readlines()
        f.close()
        pix,wav, elm = [],[],[]
        for line in lines:
                cos = line.split()
                pix.append(int(cos[1]))
                wav.append(float(cos[2]))
                elm.append(cos[3])
        vecs = []
        vec = [0]
        i = 1
        while i < len(pix):
                if pix[i] - pix[i-1] <= 8:
                        vec.append(i)
                else:
                        vecs.append(vec)
                        vec=[i]
                i+=1
        vecs.append(vec)
        f = open(inp[:11]+'iwdat','w')
        for vec in vecs:
                lout = str(len(vec))
                for i in vec:
                        lout += '\t' + str(pix[i]) + '\t' + str(wav[i])
                for i in vec:
                        lout += '\t' + elm[i]
                lout += '\n'
                f.write(lout)
        f.close()
        ii+=1
                
        



"""Instrument pipeline registry."""

INSTRUMENTS = {
    'feros': 'ferospipe_fp',
    'harps': 'harpspipe',
    'coralie': 'coraliepipe',
    'espadons': 'espadonspipe',
    'uves': 'uvespipe',
    'fideos': 'fideospipe',
    'fies': 'fiespipe',
    'hires': 'hirespipe',
    'dupont': 'dupontpipe',
    'mike': 'mikepipe',
    'pfs': 'pfspipe',
    'cafe': 'cafepipe',
    'pucheros': 'pucherospipe',
    'arces': 'arcespipe',
    'vbt': 'vbtpipe',
}


def get_pipeline_module(instrument):
    """Import and return the pipeline module for the given instrument."""
    if instrument not in INSTRUMENTS:
        raise ValueError(f"Unknown instrument: {instrument}. "
                         f"Available: {', '.join(sorted(INSTRUMENTS))}")
    module_name = INSTRUMENTS[instrument]
    import importlib
    return importlib.import_module(f'ceres3.instruments.{module_name}')


def list_instruments():
    """Return sorted list of supported instrument names."""
    return sorted(INSTRUMENTS.keys())



from astropy.io import fits as pyfits
import argparse
import os
import spfr
import glob
import numpy as np
import pickle

parser = argparse.ArgumentParser()
parser.add_argument('directorio')
parser.add_argument('-pars',default='6000,4.5,0.0,100')
parser.add_argument('-model_path',default='../../data/COELHO2014/')

args   = parser.parse_args()
dirin  = args.directorio
pars   = args.pars
model_path  = args.model_path

teff  = float(pars.split(',')[0])
logg  = float(pars.split(',')[1])
feh   = float(pars.split(',')[2])
vsini = float(pars.split(',')[3])

try:
	sc = pyfits.getdata(dirin)
	fits = [dirin]
	dirin = dirin.split('/')
	dirin = dirin[:-1]
	dt = ''
	for fn in dirin:
		dt += fn+'/'
	dirin = dt
except:
	fits = glob.glob(dirin + '/*fits')

os.system('mkdir '+dirin+'/FR')


for fit in fits:
	print('RV computation of file ' + fit + ' whith model:' + pars)

	sc = pyfits.getdata(fit)
	hd = pyfits.getheader(fit)

	I = np.where(sc[0,:,0]<5200)[0]
	npix = sc.shape[2]
	SNR_5130 = np.median(sc[8,I[0],int(0.5*npix)-100:int(0.5*npix)+100] )
	if SNR_5130 < 1.:
		SNR_5130 = 1.
	pix = (sc[0,0,0]/(sc[0,0,1]-sc[0,0,0]))/float(hd['RESOL'])
	SNR_5130_R = np.around(SNR_5130*np.sqrt(pix))

	p1gau,vels,xc_av,XCmodelgau = spfr.RVforFR(sc[0],sc[5], teff=teff, logg=logg, feh=feh, vsini=vsini, model_path=model_path,vstep=5.)

	A = 0.11081
	B = 0.0016
	D = 0.32815
	C = 0.00453

	RVerr =  B + ( 1.6 + 0.2 * p1gau[2] ) * A / np.round(SNR_5130)
	BSerr =  D / float(np.round(SNR_5130)) + C

	RVerr =  B + (1.6+0.2*p1gau[2]*0.5)*A/np.round(SNR_5130)
	depth_fact = 1. + p1gau[0]/(p1gau[2]*np.sqrt(2*np.pi))
	if depth_fact < 0.6:
		depth_fact = 0.6

	if depth_fact >= 1.:
		RVerr2 = -999.000
	else:
		depth_fact = (1 - 0.6) / (1 - depth_fact)
		RVerr2 = RVerr * depth_fact

	RVerr2 = np.around(RVerr2,4)
	BSerr  = np.around(BSerr,4)

	print('\tRV = ',p1gau[1], '+/-', RVerr2, 'km/s')

	p1gau_m = p1gau
	XCmodel = XCmodelgau
	xc_dict = {'p1gau':p1gau, 'vels':vels, 'xc_av':xc_av,'XCmodelgau':XCmodelgau}
	ccf_pdf = dirin + '/FR/' + fit.split('/')[-1][:-4] + '_XC_FR.pdf'
	pkl_xc  = dirin + '/FR/' + fit.split('/')[-1][:-4] + '_XC_FR.pkl'
	pickle.dump( xc_dict, open( pkl_xc, 'wb' ) )
	spfr.plot_CCF_FR(xc_dict,path=ccf_pdf)
	SP2 = spfr.calc_bss2(vels,xc_av,p1gau)

	print('\tBS = ',SP2, '+/-', BSerr,'km/s')

	line_out = "%-15s %18.8f %9.4f %7.4f %9.3f %5.3f   %s   ceres_FR   %8d %6d %5.2f %5.2f %5.1f %4.2f %5.2f %6.1f %4d %s\n"%\
               (hd['HIERARCH TARGET NAME'], 2400000.5 + float(hd['HIERARCH MBJD']), p1gau[1], RVerr2, SP2, BSerr, hd['INST'], int(hd['RESOL']), teff, logg, feh, vsini, xc_av.min(), p1gau[2],\
		       hd['TEXP (s)'], SNR_5130, ccf_pdf)

	i = 0
	isin = False
	if os.access(dirin+'/FR/results_FR.txt',os.F_OK):
		f = open(dirin+'/FR/results_FR.txt','r')
		lines = f.readlines()
		for line in lines:
			cos = line.split()
			if cos[1] == line_out.split()[1]:
				lines[i] = line_out
				isin = True
				break
			i+=1
		if not isin:
			lines.append(line_out)
	else:
		lines = [line_out]

	f = open(dirin+'/FR/results_FR.txt','w')
	for line in lines:
		f.write(line)
	f.close()



"""Command-line interface for CERES3 spectral reduction."""

import argparse
import sys


def cmd_reduce(args):
    """Run a full instrument reduction pipeline."""
    from ceres3.instruments import get_pipeline_module
    pipeline = get_pipeline_module(args.instrument)

    # Reconstruct argv for backward compat with module-level argparse
    sys.argv = [f'ceres3-{args.instrument}', args.input_dir]
    if args.dirout:
        sys.argv.extend(['-dirout', args.dirout])
    sys.argv.extend(['-npools', str(args.npools)])
    sys.argv.extend(['-nsigmas', str(args.nsigmas)])
    if args.is_calib:
        sys.argv.append('-is_calib')
    if args.calib_dir:
        sys.argv.extend(['-calib_dir', args.calib_dir])
    if args.do_class:
        sys.argv.append('-do_class')
    if args.o2do != 'all':
        sys.argv.extend(['-o2do', args.o2do])

    import importlib
    importlib.reload(pipeline)


def cmd_activity(args):
    """Compute activity indicators on already-reduced spectra."""
    import os
    import glob
    import numpy as np
    from astropy.io import fits as pyfits
    from ceres3.utils.activity import compute_activity

    input_dir = args.input_dir.rstrip('/')
    proc_dir = os.path.join(input_dir, 'proc') if os.path.isdir(os.path.join(input_dir, 'proc')) else input_dir

    # Find all reduced spectrum cubes
    patterns = [os.path.join(proc_dir, '*_sp.fits')]
    fits_files = sorted(set(f for p in patterns for f in glob.glob(p)))

    if not fits_files:
        print(f"No *_sp.fits files found in {proc_dir}")
        sys.exit(1)

    print(f"Found {len(fits_files)} spectra in {proc_dir}")
    print(f"{'File':<60} {'S':>7} {'logRHK':>7} {'Ha':>7} {'Hb':>7} {'HeI':>7} {'NaI':>7} {'CaIRT':>7} {'MgIb':>7}")
    print("-" * 130)

    results = []
    for fpath in fits_files:
        fname = os.path.basename(fpath)
        try:
            hdul = pyfits.open(fpath)
            spec = hdul[0].data
            header = hdul[0].header

            # Get Teff if available
            teff = header.get('TEFF', None)
            if teff is not None and teff <= 0:
                teff = None

            instrument = header.get('INST', args.instrument or 'feros').lower()

            # Output 1D path
            out_1d = fpath.replace('_sp.fits', '_1d.fits') if args.save_1d else None

            activity = compute_activity(spec, instrument=instrument,
                                        output_1d_path=out_1d, teff=teff)

            # Write activity values back to FITS header
            if args.update_fits:
                with pyfits.open(fpath, mode='update') as hdu_update:
                    for key, val in activity.items():
                        if key == 'spectrum_1d_path':
                            continue
                        try:
                            hdu_update[0].header[key.upper()] = np.around(val, 6)
                        except Exception:
                            pass
                    hdu_update.flush()

            a = activity
            print(f"{fname:<60} {a['s_index']:>7.4f} {a['log_rhk']:>7.3f} {a['halpha']:>7.4f} {a['hbeta']:>7.4f} {a['hei']:>7.4f} {a['nai_d1d2']:>7.4f} {a['ca_irt']:>7.4f} {a['mg_ib']:>7.4f}")
            results.append({'file': fname, **activity})
            hdul.close()

        except Exception as e:
            print(f"{fname:<60} ERROR: {e}")

    # Write summary file
    if args.output:
        outpath = args.output
    else:
        outpath = os.path.join(proc_dir, 'activity.txt')

    with open(outpath, 'w') as f:
        header_line = "# file bjd s_index s_index_err log_rhk halpha halpha_err hbeta hbeta_err hei hei_err nai_d1d2 nai_d1d2_err ca_irt ca_irt_err mg_ib mg_ib_err\n"
        f.write(header_line)
        for r in results:
            bjd = r.get('bjd', 0.0)
            f.write(f"{r['file']} {r['s_index']:.6f} {r['s_index_err']:.6f} {r['log_rhk']:.4f} "
                    f"{r['halpha']:.6f} {r['halpha_err']:.6f} "
                    f"{r['hbeta']:.6f} {r['hbeta_err']:.6f} "
                    f"{r['hei']:.6f} {r['hei_err']:.6f} "
                    f"{r['nai_d1d2']:.6f} {r['nai_d1d2_err']:.6f} "
                    f"{r['ca_irt']:.6f} {r['ca_irt_err']:.6f} "
                    f"{r['mg_ib']:.6f} {r['mg_ib_err']:.6f}\n")

    print(f"\nResults written to {outpath}")
    if args.save_1d:
        n_1d = sum(1 for r in results if r.get('spectrum_1d_path'))
        print(f"1D spectra saved: {n_1d}/{len(results)}")


def main():
    from ceres3.instruments import list_instruments

    parser = argparse.ArgumentParser(
        prog='ceres3',
        description='CERES3: Echelle spectrograph data reduction pipeline',
    )
    subparsers = parser.add_subparsers(dest='command', help='Command to run')

    # ── reduce ────────────────────────────────────────────────────────
    p_reduce = subparsers.add_parser('reduce', help='Run instrument reduction pipeline')
    p_reduce.add_argument('instrument', choices=list_instruments())
    p_reduce.add_argument('input_dir')
    p_reduce.add_argument('-o', '--output-dir', default=None, dest='dirout')
    p_reduce.add_argument('--npools', type=int, default=4)
    p_reduce.add_argument('--nsigmas', type=float, default=5.0)
    p_reduce.add_argument('--is-calib', action='store_true')
    p_reduce.add_argument('--calib-dir', default=None)
    p_reduce.add_argument('--do-class', action='store_true')
    p_reduce.add_argument('--target', default='all', dest='o2do')

    # ── activity ──────────────────────────────────────────────────────
    p_activity = subparsers.add_parser('activity',
        help='Compute activity indicators on already-reduced spectra')
    p_activity.add_argument('input_dir',
        help='Directory containing reduced *_sp.fits files (or parent with proc/ subdir)')
    p_activity.add_argument('--instrument', default=None,
        help='Override instrument name (default: read from FITS header)')
    p_activity.add_argument('-o', '--output', default=None,
        help='Output file for activity summary (default: proc/activity.txt)')
    p_activity.add_argument('--save-1d', action='store_true',
        help='Save merged 1D rest-frame spectra as *_1d.fits')
    p_activity.add_argument('--update-fits', action='store_true',
        help='Write activity values back into the FITS headers')

    args = parser.parse_args()

    if args.command == 'reduce':
        cmd_reduce(args)
    elif args.command == 'activity':
        cmd_activity(args)
    else:
        parser.print_help()
        sys.exit(1)


if __name__ == '__main__':
    main()



import os
f = open('/data/echelle/harps/list.txt','r')
lines = f.readlines()
rv,rve,jd,refs = [],[],[],[]
for line in lines:
        path = line[:-1]
        #line = 'python ferospipe.py '+path+' -npools 16 -do_class -avoid_plot -o2do HD157347 -reffile /data/echelle/feros/reffile.txt'
        line = 'python harpspipe.py '+path+' -npools 10 -do_class -avoid_plot'
        os.system(line)
        """
        pr = path[:-1]+'_red/proc/results.txt'

        try:
                fo = open(pr,'r')
                lines2 = fo.readlines()
                for line2 in lines2:
                        cos = line2.split()
                        if float(cos[16])>50 and cos[0]=='HD157347':
                                print(line2[:-1])
                                rv.append(float(cos[2]))
                                rve.append(float(cos[3]))
                                jd.append(float(cos[1]))
                                st = path[:-1]
                                st = st.split('/')[-1]
                                mo = float(st[4:6])
                                da = float(st[6:])
                                ref = mo + da/30.5
                                refs.append(ref)
        except:
                print('bad')
        
        
jd,rv,refs = np.array(jd),np.array(rv),np.array(refs)
I = np.where(refs<20)[0]
print(np.sqrt(np.var(rv[I])))
errorbar(refs,rv,yerr=rve,fmt='ro')
show()
"""



import matplotlib
matplotlib.use("Agg")
from astropy.io import fits as pyfits
import numpy as np
import scipy
import glob
import os
import matplotlib.pyplot as plt
import sys
from ceres3.utils import globalutils as GLOBALutils

import pycurl

def FileClassify(dir, log,binning):
    """
    Classifies all files in a directory and writes a night log of science images
    """

    # define output lists
    thars      = []
    biases     = []
    quartz     = []
    iodines    = []
    science    = []
    obnames    = []
    exptimes   = []
    thar_dates = []

    f = open(log,'w')

    bad_files = []
    if os.access(dir+'bad_files.txt',os.F_OK):
        bf = open(dir+'bad_files.txt')
        linesbf = bf.readlines()
        for line in linesbf:
            bad_files.append(dir+line[:-1])
        bf.close()
    
    all_files = glob.glob(dir+"/pfs*fits")

    for archivo in all_files:
        dump = False
        for bf in bad_files:
            if archivo == bf:
                dump = True
                break
    
        h = pyfits.open(archivo)
        if dump == False and h[0].header['BINNING']==binning:
            if h[0].header['EXPTYPE'] == 'Object' and h[0].header['OBJECT'] != 'ThAr' and h[0].header['OBJECT'] != 'WideFlat':
                science.append(archivo)
                obname  = h[0].header['OBJECT']
                ra      = h[0].header['RA']
                delta   = h[0].header['DEC']
                airmass = h[0].header['AIRMASS']
                texp    = h[0].header['EXPTIME']
                date    = h[0].header['UT-DATE']
                hour    = h[0].header['UT-TIME']
                obnames.append( obname )
                exptimes.append( texp ) 
                line = "%-15s %10s %10s %8.2f %4.2f %8s %8s %s\n" % (obname, ra, delta, texp, airmass, date, hour, archivo)
                f.write(line)
            elif h[0].header['EXPTYPE'] == 'Bias':
                print(h[0].data.shape)
                biases.append(archivo)
            elif h[0].header['EXPTYPE'] == 'Quartz' or h[0].header['EXPTYPE'] == 'Flat':
                if h[0].header['IODINE'] == 'Window':
                    quartz.append(archivo)
                elif h[0].header['IODINE'] == 'I-Cell':
                    iodines.append(archivo)
            elif h[0].header['EXPTYPE'] == 'ThAr' or h[0].header['OBJECT'] == 'ThAr':
                thars.append(archivo)
                mjd, mjd0 = mjd_fromheader(h)
                thar_dates.append( mjd )

    f.close()
    return biases, quartz, iodines, science, thars, thar_dates, obnames, exptimes

def mjd_fromheader(h):
    """
    return modified Julian date from header
    """
    
    datetu = h[0].header['UT-DATE'] 
    ut     = h[0].header['UT-TIME']
    mjd0,mjd,i = GLOBALutils.iau_cal2jd(int(datetu[0:4]),int(datetu[5:7]),int(datetu[8:]))
    ut = float(ut[:2]) + float(ut[3:5])/60. + float(ut[6:])/3600.
    mjd_start = mjd + ut/24.0
    secinday = 24*3600.0
    fraction = .5
    texp     = h[0].header['EXPTIME'] #sec
    mjd = mjd_start + (fraction * texp) / secinday

    return mjd, mjd0

def MedianCombine(ImgList, bs, os, bias = 0.):
    """

    Median combine a list of images

    """

    n = len(ImgList)
    #if n==0:
    #    raise ValueError("empty list provided!")
    if n == 0:
        print("\t\t\tWarning: 0 biases")
        return 0, 0, 0
    h = pyfits.open(ImgList[0])[0]
    d = h.data
    d = OverscanTrim(d,bs,os)
    d -= bias
    
    factor = 1.25
    if (n < 3):
        factor = 1

    ronoise = factor * h.header['ENOISE'] / np.sqrt(n)
    gain    = h.header['EGAIN']

    
    if (n == 1):
        return d, ronoise, gain
    else:
        for i in range(n-1):
            h = pyfits.open(ImgList[i+1])[0]
            d2 = OverscanTrim(h.data,bs,os)
            d = np.dstack( (d ,OverscanTrim(h.data,bs,os) -bias ))
        return np.median(d,axis=2), ronoise, gain

def OverscanTrim(d,bs,ts):
    """
    Overscan correct and Trim a refurbished PFS image
    """
    # bias has no significant structure, so a single median suffices, I think
    # overscan = [0:49] [2097:2145]
    #bs = [0,2032,2160]
    #ts = [1,2032,2160]
    if bs[0] == 0:
        bss = d[bs[1]:bs[2],:]
        bss = np.median(bss,axis=0)[:ts[1]]
        bss = np.tile(bss,bs[1])
        bss = np.reshape(bss,[bs[1],ts[1]])
        newdata = d[:bs[1],:ts[1]] - bss
    else:
        bss = d[:,bs[1]:bs[2]]
        bss = np.median(bss,axis=1)[:ts[1]]
        bss = np.tile(bss,ts[1])
        bss = np.reshape(bss,[ts[1],bs[1]]).T
        newdata = d[:ts[1],:bs[1]] - bss

    return newdata



from astropy.io import fits as pyfits
import argparse
import os
import spfr
import glob

parser = argparse.ArgumentParser()
parser.add_argument('directorio')
parser.add_argument('-model_path',default='../../data/COELHO2014/')
parser.add_argument('-npools', default=1)
parser.add_argument('-fixG', default=-1)

args   = parser.parse_args()
dirin  = args.directorio
model_path  = args.model_path
npools = int(args.npools)
fixG = float(args.fixG)

try:
	sc = pyfits.getdata(dirin)
	fits = [dirin]
	dirin = dirin.split('/')
	dirin = dirin[:-1]
	dt = ''
	for fn in dirin:
		dt += fn+'/'
	dirin = dt
except:
	fits = glob.glob(dirin + '/*fits')

os.system('mkdir '+dirin+'/FR')
print('\n\n')
for fit in fits:
	print('Estimation of atmospheric parameters for ' + fit + '...')

	sc = pyfits.getdata(fit)
	hd = pyfits.getheader(fit)
	teff, logg, feh, vsini = spfr.get_pars_fr(sc[0],sc[5],model_patht=model_path,npools=npools,fixG=fixG)
	print('\tTeff=', teff, 'K')
	print('\tlog(g)=', logg, 'dex')
	print('\t[Fe/H]=', feh, 'dex')
	print('\tvsini=', vsini, 'km/s\n')


	line_out = "%-15s %18.8f %5.2f %5.2f %5.1f %4.2f %s\n"%(hd['HIERARCH TARGET NAME'], 2400000.5 + float(hd['HIERARCH MBJD']), teff, logg, feh, vsini, fit)

	i    = 0
	isin = False
	if os.access(dirin+'/FR/pars_FR.txt',os.F_OK):
		f = open(dirin+'/FR/pars_FR.txt','r')
		lines = f.readlines()
		for line in lines:
			cos = line.split()
			if cos[1] == line_out.split()[1]:
				lines[i] = line_out
				isin = True
				break
			i+=1
		if not isin:
			lines.append(line_out)
	else:
		lines = [line_out]

	f = open(dirin+'/FR/pars_FR.txt','w')
	for line in lines:
		f.write(line)
	f.close()



import matplotlib
matplotlib.use("Agg") 
from astropy.io import fits as pyfits
import numpy as np
from numpy import median,sqrt,array,exp
import scipy
from scipy import signal,special,optimize,interpolate
import scipy.special as sp
import copy
import glob
import os
import matplotlib.pyplot as plt
import sys
from ceres3.utils import globalutils as GLOBALutils


def FileClassify(diri, log,mode='so',amps='a'):
        """
        Classifies all files in a directory and writes a night log of science images
        """
        if mode == 'so':
                md = 'star only'
        elif mode == 'ss':
                md = 'star+sky'
        else:
                md = 'Polarimetry'

        if amps == 'a':
                samps = 'a'
        elif amps == 'ab':
                samps = 'a,b'

        # define output lists
        simThAr_sci    = []
        simSky_sci     = []
        biases         = []
        bias_dates     = []
        flat_dates     = []
        flats          = []
        ThAr_ref       = []
        ThAr_ref_dates = []
        mode           = []

        f = open(log,'w')
        bad_files = []
        if os.access(diri+'bad_files.txt',os.F_OK):
                bf = open(diri+'bad_files.txt')
                linesbf = bf.readlines()
                for line in linesbf:
                        bad_files.append(diri+line[:-1])
                bf.close()

        all_files = glob.glob(diri+"*fits")

        jj = 0

        for archivo in all_files:
                jj+=1
                dump = False
                for bf in bad_files:
                        if archivo == bf:
                                dump = True
                                break

                if not dump:
                        h = pyfits.open(archivo)
                        #print archivo, h[0].header['EXPTYPE'], h[0].header['INSTMODE']
                        if md in h[0].header['INSTMODE'] and samps == h[0].header['AMPLIST']:
                                print(archivo, h[0].header['EXPTYPE'], h[0].header['INSTMODE'], h[0].data.shape, h[0].header['AMPLIST'])
                                if h[0].header['EXPTYPE'] == 'OBJECT':
                                        simThAr_sci.append(archivo)
                                        obname = h[0].header['OBJNAME']
                                        ra     = h[0].header['OBJRA']
                                        delta  = h[0].header['OBJDEC']
                                        airmass= h[0].header['AIRMASS']
                                        texp   = h[0].header['EXPTIME']
                                        date   = h[0].header['DATE-OBS'] + h[0].header['UTC-OBS']
                                        line = "%-15s %10s %10s %8.2f %4.2f %s %8s %s\n" % (obname, ra, delta, texp, airmass, h[0].header['EXPTYPE'], date, archivo)
                                        f.write(line)

                                elif h[0].header['EXPTYPE'] == 'BIAS':
                                        biases.append(archivo)
                                        mjd,mjd0 = mjd_fromheader(h)
                                        bias_dates.append(mjd)

                                elif h[0].header['EXPTYPE'] == 'FLAT':
                                        flats.append(archivo)
                                        mjd,mjd0 = mjd_fromheader(h)
                                        flat_dates.append(mjd)

                                elif h[0].header['EXPTYPE'] == 'COMPARISON':
                                        mjd, mjd0 = mjd_fromheader(h)
                                        ThAr_ref.append(archivo) 
                                        ThAr_ref_dates.append( mjd )

        f.close()
        biases, bias_dates = np.array(biases), np.array(bias_dates)
        flats, flat_dates  = np.array(flats), np.array(flat_dates)
        IS = np.argsort(bias_dates)
        biases, bias_dates = biases[IS], bias_dates[IS]
        IS = np.argsort(flat_dates)
        flats, flat_dates = flats[IS], flat_dates[IS]

        return biases, flats, ThAr_ref, simThAr_sci, ThAr_ref_dates

def mjd_fromheader(h):
    """
    return modified Julian date from header
    """
    
    datetu = h[0].header['DATE-OBS']
    timetu = h[0].header['UTC-OBS'].split(':')

    mjd0,mjd,i = GLOBALutils.iau_cal2jd(int(datetu[:4]),int(datetu[5:7]),int(datetu[8:10]))

    ho = int(timetu[0])
    mi = int(timetu[1])
    se = float(timetu[2])
    ut = float(ho) + float(mi)/60.0 + float(se)/3600.0
    mjd_start = mjd + ut/24.0

    secinday = 24*3600.0
    fraction = 0.5
    texp     = h[0].header['EXPTIME'] #sec

    mjd = mjd_start + (fraction * texp) / secinday

    return mjd, mjd0

def MedianCombine(ImgList, zero_bo=False, zero='MasterBias.fits'):
        """
        Median combine a list of images
        """
        if zero_bo:
                BIAS = pyfits.getdata(zero)

        n = len(ImgList)
        if n==0:
                raise ValueError("empty list provided!")

        h = pyfits.open(ImgList[0])[0]
        d = h.data
        d = OverscanTrim(d)
        if zero_bo:
                d -= BIAS
                d = np.round(d).astype('int')
        factor = 1.25
        if (n < 3):
                factor = 1
        ronoise = factor * h.header['RDNOISEA'] / np.sqrt(n)
        gain    = h.header['GAINA']
        if (n == 1):
                return d, ronoise, gain
        else:
                for i in range(n-1):
                        h = pyfits.open(ImgList[i+1])[0]
                        ot = OverscanTrim(h.data)
                        if zero_bo:
                                d = np.dstack((d,np.round((ot - BIAS)).astype('int')))
                        else:
                                d = np.dstack((d,np.round(ot).astype('int')))
        return np.median(d,axis=2), ronoise, gain

def OverscanTrim(d):
    """
    Overscan correct and Trim a refurbished FEROS image
    """
    os = d[:,2060:]
    overscan = np.median(os,axis=1)[:4608]
    newdata = np.zeros(d[:4608,:2049].shape)
    for i in range(len(overscan)):
        newdata[i,:] = d[i,:2049] - overscan[i]
    return newdata

def get_them(sc,exap,ncoef,mode=1,shift=0.):
        def gauss(params,x):
                amp1 = params[0]
                med1 = params[1]
                sig1 = params[2]
                g1 = amp1 * np.exp(-0.5*((x-med1)/sig1)**2)
                return g1
        def res_gauss(params,g,x):
                return g-gauss(params,x)

        def gauss2(params,x):
                amp1 = params[0]
                amp2 = params[1]
                med1 = params[2]
                med2 = params[3]
                sig1 = params[4]
                sig2 = params[5]
                g1 = amp1 * np.exp(-0.5*((x-med1)/sig1)**2)
                g2 = amp2 * np.exp(-0.5*((x-med2)/sig2)**2)
                return g1 + g2

        def res_gauss2(params,g,x):
                return g-gauss2(params,x)

        def fit_drift(ref,dif,oref):
                coef  = np.polyfit(ref,dif,2)

                residuals = dif - np.polyval(coef,ref)
                rms = np.sqrt(np.var(residuals))
                I = np.where(np.absolute(residuals)>3*rms)[0]
                cond = True
                if len(I)==0:
                        cond = False
                while cond:
                        im  = np.argmax(np.absolute(residuals))
                        dif = np.delete(dif,im)
                        ref = np.delete(ref,im)
                        coef = np.polyfit(ref,dif,2)
                        residuals = dif - np.polyval(coef,ref)
                        rms = np.sqrt(np.var(residuals))
                        I = np.where(np.absolute(residuals)>3*rms)[0]
                        if len(I)==0:
                                cond = False

                cdif = np.polyval(coef,oref)
                ref = oref + cdif
                return ref
        
        sc_or = sc.copy()  
        medc = int(.5*sc.shape[1])
        d = np.median(sc[:,medc-exap:medc+exap+1],axis=1)
        #plot(d)
        ejx = np.arange(len(d))

        ref = []
        cents = np.loadtxt('ref_ords.txt').astype('int') - shift
        #plot(d)
        #plot(ejx[cents[:,0]],d[cents[:,0]],'ro')
        #plot(ejx[cents[:,1]],d[cents[:,1]],'go')
        #show()

        posO, posC = cents[:,0],cents[:,1]
        if mode == 0:
                posO = cents[:,2]
                posC = cents[:,2]
        refO,refC = [],[]
        exap2 = exap + .2*exap
        dev   = exap2/4.
        for i in range(len(posO)):
                if posO[i]-exap2 < 0:
                        x = ejx[:posC[i]+exap2+1]
                        y = d[:posC[i]+exap2+1]
                elif posC[i]+exap2+1 > len(d):
                    x = ejx[posO[i]-exap2:]
                    y = d[posO[i]-exap2:]
                else:   
                        x = ejx[posO[i]-exap2:posC[i]+exap2+1]
                        y = d[posO[i]-exap2:posC[i]+exap2+1]
                y -= y.min()
                tx1 = np.arange(x[0]-dev,x[0],1)
                tx2 = np.arange(x[-1]+1,x[-1]+dev+1,1)
                ty1 = np.zeros(len(tx1))
                ty2 = np.zeros(len(tx2))
                x = np.hstack((tx1,x,tx2))
                y = np.hstack((ty1,y,ty2))

                midi = int(0.5*len(x))
                
                if mode == 1:
                        guess = [np.max(y[:midi]),np.max(y[midi:]),x[0]+np.argmax(y[:midi]),x[0]+midi+np.argmax(y[midi:]),dev,dev]
                        p, success =  scipy.optimize.leastsq(res_gauss2, guess, args=(y,x))
                        refO.append(p[2])
                        refC.append(p[3])
                else:
                        #plot(x,y)
                        #show()
                        #print fcds
                        guess = [np.max(y),x[0]+np.argmax(y),dev]
                        p, success =  scipy.optimize.leastsq(res_gauss, guess, args=(y,x))
                        refO.append(p[1])
                        refC.append(p[1])

        refO = np.array(refO)
        refOt = np.around(refO).astype('int')
        matO = np.zeros((len(refO),sc.shape[1]))
        matO[:,medc] = refO
        refC = np.array(refC)
        refCt = np.around(refC).astype('int')
        matC = np.zeros((len(refC),sc.shape[1]))
        matC[:,medc] = refC

        #plot(d)
        #plot(cents[:,0],d[cents[:,2]],'ro')
        #plot(cents[:,1],d[cents[:,1]],'go')
        #plot(refOt,d[refOt],'ko')
        #plot(refCt,d[refCt],'yo')
        #show()
        #print gfd5
        i = medc -1
        while i >=0:
                d = sc[:,i]
                #if i<1000:
                #       plot(d)
                j = 0
                posO = np.around(refO).astype('int')
                posC = np.around(refC).astype('int')
                trefO,trefC = [],[]
                exap2 = exap + .2*exap
                dev   = exap2/4.

                while j < len(posO):
                        #print j
                        if posO[j]-exap2 < 0:
                                x = ejx[:posC[j]+exap2+1]
                                y = d[:posC[j]+exap2+1]
                        elif posC[j]+exap2+1 > len(d):
                                x = ejx[posO[j]-exap2:]
                                y = d[posO[j]-exap2:]
                        else:   
                                x = ejx[posO[j]-exap2:posC[j]+exap2+1]
                                y = d[posO[j]-exap2:posC[j]+exap2+1]
                        if len(x) < 7:
                                trefO.append(refO[j])
                                trefC.append(refC[j])
                        else:
                                y -= y.min()
                                tx1 = np.arange(x[0]-dev,x[0],1)
                                tx2 = np.arange(x[-1]+1,x[-1]+dev+1,1)
                                ty1 = np.zeros(len(tx1))
                                ty2 = np.zeros(len(tx2))
                                x = np.hstack((tx1,x,tx2))
                                y = np.hstack((ty1,y,ty2))
                                midi = int(0.5*len(x))
                                if mode == 1:
                                        guess = [np.max(y[:midi]),np.max(y[midi:]),x[0]+np.argmax(y[:midi]),x[0]+midi+np.argmax(y[midi:]),dev,dev]
                                        p, success =  scipy.optimize.leastsq(res_gauss2, guess, args=(y,x))
                                        trefO.append(p[2])
                                        trefC.append(p[3])
                                else:
                                        guess = [np.max(y),x[0]+np.argmax(y),dev]
                                        p, success =  scipy.optimize.leastsq(res_gauss, guess, args=(y,x))
                                        trefO.append(p[1])
                                        trefC.append(p[1])

                        j+=1

                orefO = refO.copy()
                trefO = np.array(trefO)
                difO = trefO-refO
                refO = fit_drift(refO,difO,orefO)
                matO[:,i] = refO
                orefC = refC.copy()
                trefC = np.array(trefC)
                difC = trefC-refC
                refC = fit_drift(refC,difC,orefC)
                matC[:,i] = refC

                #if i<500:
                #       plot(d)
                #       I = np.where(refO<2049)[0]
                #       plot(refO[I].astype('int'),d[refO[I].astype('int')],'ko')
                #       show()
                #       print gfd

                i-=4

        refO = matO[:,medc]
        refC = matC[:,medc]

        i = medc +1
        while i < sc.shape[1]:
                d = sc[:,i]
                j = 0
                posO = np.around(refO).astype('int')
                posC = np.around(refC).astype('int')
                trefO,trefC = [],[]
                exap2 = exap + .2*exap
                dev   = exap2/4.

                while j < len(posO):
                        #print j
                        if posO[j]-exap2 < 0:
                                x = ejx[:posC[j]+exap2+1]
                                y = d[:posC[j]+exap2+1]
                        elif posC[j]+exap2+1 > len(d):
                                x = ejx[posO[j]-exap2:]
                                y = d[posO[j]-exap2:]
                        else:   
                                x = ejx[posO[j]-exap2:posC[j]+exap2+1]
                                y = d[posO[j]-exap2:posC[j]+exap2+1]
                        if len(x) < 7:
                                trefO.append(refO[j])
                                trefC.append(refC[j])
                        else:
                                y -= y.min()
                                tx1 = np.arange(x[0]-dev,x[0],1)
                                tx2 = np.arange(x[-1]+1,x[-1]+dev+1,1)
                                ty1 = np.zeros(len(tx1))
                                ty2 = np.zeros(len(tx2))
                                x = np.hstack((tx1,x,tx2))
                                y = np.hstack((ty1,y,ty2))
                                midi = int(0.5*len(x))
                                if mode == 1:
                                        guess = [np.max(y[:midi]),np.max(y[midi:]),x[0]+np.argmax(y[:midi]),x[0]+midi+np.argmax(y[midi:]),dev,dev]
                                        p, success =  scipy.optimize.leastsq(res_gauss2, guess, args=(y,x))
                                        trefO.append(p[2])
                                        trefC.append(p[3])
                                else:
                                        guess = [np.max(y),x[0]+np.argmax(y),dev]
                                        p, success =  scipy.optimize.leastsq(res_gauss, guess, args=(y,x))
                                        trefO.append(p[1])
                                        trefC.append(p[1])
                        j+=1

                orefO = refO.copy()
                trefO = np.array(trefO)
                difO = trefO-refO
                refO = fit_drift(refO,difO,orefO)
                matO[:,i] = refO

                orefC = refC.copy()
                trefC = np.array(trefC)
                difC = trefC-refC
                refC = fit_drift(refC,difC,orefC)
                matC[:,i] = refC

                i+=4

        #imshow(sc_or,vmax=5000)
        for i in range(matO.shape[0]):
                y = matO[i]
                x = np.arange(len(y))
                I = np.where(y!=0)[0]
                x,y = x[I],y[I]
                coef = np.polyfit(x,y,ncoef)
                #plot(x,y,'r')
                residuals = y - np.polyval(coef,x)
                rms = np.sqrt(np.var(residuals))
                I = np.where(np.absolute(residuals)>3*rms)[0]
                cond = True
                if len(I)==0:
                        cond = False
                while cond:
                        im = np.argmax(np.absolute(residuals))
                        x = np.delete(x,im)
                        y = np.delete(y,im)
                        coef = np.polyfit(x,y,ncoef)
                        residuals = y - np.polyval(coef,x)
                        rms = np.sqrt(np.var(residuals))
                        I = np.where(np.absolute(residuals)>3*rms)[0]
                        if len(I)==0:
                                cond = False
                if i == 0:
                        acoefsO = coef
                else:
                        acoefsO = np.vstack((acoefsO,coef))
                #plot(np.polyval(coef,np.arange(len(matO[i]))),'g')
        #show()

        if mode == 1:
                for i in range(matC.shape[0]):
                        y = matC[i]
                        x = np.arange(len(y))
                        I = np.where(y!=0)[0]
                        x,y = x[I],y[I]
                        coef = np.polyfit(x,y,ncoef)
                        #plot(x,y,'r')
                        residuals = y - np.polyval(coef,x)
                        rms = np.sqrt(np.var(residuals))
                        I = np.where(np.absolute(residuals)>3*rms)[0]
                        cond = True
                        if len(I)==0:
                                cond = False
                        while cond:
                                im = np.argmax(np.absolute(residuals))
                                x = np.delete(x,im)
                                y = np.delete(y,im)
                                coef = np.polyfit(x,y,ncoef)
                                residuals = y - np.polyval(coef,x)
                                rms = np.sqrt(np.var(residuals))
                                I = np.where(np.absolute(residuals)>3*rms)[0]
                                if len(I)==0:
                                        cond = False
                        if i == 0:
                                acoefsC = coef
                        else:
                                acoefsC = np.vstack((acoefsC,coef))
                        #plot(np.polyval(coef,np.arange(len(matC[i]))),'b')
                #show()
                #print gfds
                #print acoefs.shape
                return acoefsO, acoefsC
        else:
                return acoefsO



import matplotlib
matplotlib.use("Agg")
from astropy.io import fits as pyfits
import numpy as np
from numpy import median, sqrt, array, exp
import scipy
from scipy import signal, special, optimize, interpolate
import scipy.special as sp
import copy
import glob
import os
import matplotlib.pyplot as plt
import sys
from ceres3.utils import globalutils as GLOBALutils


import statsmodels.api as sm
lowess = sm.nonparametric.lowess

def gauss2(params, x):
    amp1 = params[0]
    amp2 = params[1]
    med1 = params[2]
    med2 = params[3]
    sig1 = params[4]
    sig2 = params[5]
    g1 = amp1 * np.exp(-0.5*((x-med1)/sig1)**2)
    g2 = amp2 * np.exp(-0.5*((x-med2)/sig2)**2)
    return g1 + g2

def res_gauss2(params, g, x):
    return g-gauss2(params, x)

def get_RG(h):
    if h['HIERARCH ESO DET READ MODE'] == 'normal':
        ron = 5.1
        gain = 3.2
    elif h['HIERARCH ESO DET READ MODE'] == 'slow':
        ron = 3.0
        gain = 1.0
    else:
        ron = 7.2
        gain = 2.7
    return ron, gain

def MedianCombine(ImgList, zero_bo=False, zero='MasterBias.fits'):
    """
    Median combine a list of images
    """
    if zero_bo:
        BIAS = pyfits.getdata(zero)

    n = len(ImgList)
    if n==0:
        raise ValueError("empty list provided!")

    h = pyfits.open(ImgList[0])[0]
    d = h.data
    d = OverscanTrim(d)
    d = b_col(d)
    if zero_bo:
        d -= BIAS
    d = np.round(d).astype('int')

    factor = 1.25
    if (n < 3):
        factor = 1

    #ronoise = factor * h.header['HIERARCH ESO CORA CCD RON'] / np.sqrt(n)
    #gain    = h.header['HIERARCH ESO CORA CCD GAIN']
    ronoise, gain = get_RG(h.header)
    ronoise = ronoise/np.sqrt(n)

    if (n == 1):
        return d, ronoise, gain
    else:
        for i in range(n-1):
            #print(ImgList[i+1])
            h = pyfits.open(ImgList[i+1])[0]
            ot = OverscanTrim(h.data)
            ot = b_col(ot)
            if zero_bo:
                d = np.dstack((d,np.round((ot - BIAS)).astype('int')))
            else:
                d = np.dstack((d,np.round(ot).astype('int')))
        return np.median(d,axis=2), ronoise, gain

def OverscanTrim(d):
    """
    Overscan correct and Trim a refurbished FEROS image
    """
    ps = d[:,:48]
    os = d[:,-48:]
    s = 0.5*(np.median(ps,axis=1)+np.median(os,axis=1))
    c = np.polyfit(np.arange(len(s)),s,4)
    overscan = np.polyval(c,np.arange(len(s)))
    newdata = np.zeros(d[:,50:-50].shape)
    for i in range(len(overscan)):
        newdata[i,:] = d[i,50:-50] - overscan[i]
    return newdata

def b_col(d):

    d2 = np.zeros(d.shape)
    ps = [[1675.,4097., 219.],\
          [1675.,1780., 222.],\
          [   0.,4097., 320.],\
          [   0.,4097., 326.],\
          [1616.,1617., 334.],\
          [1617.,1641., 334.],\
          [1641.,1696., 335.],\
          [1696.,4097., 335.],\
          [1617.,1641., 338.],\
          [1622.,1741., 342.],\
          [1622.,1741., 343.],\
          [ 868.,4097., 646.],\
          [1514.,2101., 843.],\
          [1501.,1691., 857.],\
          [1501.,1691., 858.],\
          [1475.,1681., 883.],\
          [1459.,4097., 900.],\
          [1454.,1501., 916.],\
          [1404.,4907.,1062.],\
          [1404.,4907.,1063.],\
          [1404.,4907.,1064.],\
          [ 608.,4097.,1298.],\
          [   0., 608.,1299.],\
          [ 608.,4097.,1299.]]
    ps = np.array(ps).astype('int')
    for i in range(len(ps)):
        d2[ps[i][0]:ps[i][1],ps[i][2]] = 1
    ej = np.arange(d.shape[1])
    for i in range(d.shape[0]):
        vec = d[i]
        I = np.where(d2[i]==0)[0]
        I2 = np.where(d2[i]==1)[0]
        if len(I2)>0:
            tck = scipy.interpolate.splrep(ej[I],vec[I],k=1)
            d[i] = scipy.interpolate.splev(ej,tck)

    return d

def gauss(params, x):
    med = params[0]
    sig = params[1]
    g = np.exp(-0.5*(x-med)*(x-med)/(sig*sig))
    return g

def res_gauss(params, g, x):
    return g-gauss(params, x)

def FileClassify(diri, log, lamp='LAMP3'):
    """
    Classifies all files in a directory and writes a night log of science images
    """

    # define output lists
    simThAr_sci = []
    simSky_sci  = []
    biases      = []
    bias_dates  = []
    flat_dates  = []
    flats       = []
    ThArNe_ref  = []
    ThAr_Ne_ref = []
    ThArNe_ref_dates  = []
    ThAr_Ne_ref_dates = []
    darks       = []
    dark_times  = []

    f = open(log,'w')
    bad_files = []
    if os.access(diri+'bad_files.txt',os.F_OK):
        bf = open(diri+'bad_files.txt')
        linesbf = bf.readlines()
        for line in linesbf:
            bad_files.append(diri+line[:-1])
        bf.close()

    all_files = glob.glob(diri+"*fits")

    jj = 0
    for archivo in all_files:

        jj+=1
        dump = False
        for bf in bad_files:
            if archivo == bf:
                dump = True
                break

        if not dump:
            h = pyfits.open(archivo)
            print(archivo, h[0].header['HIERARCH ESO DPR TYPE'])

            if h[0].header['HIERARCH ESO DPR TYPE'] == 'OBJECT,WAVE' or h[0].header['HIERARCH ESO DPR TYPE'] == 'VELOC,WAVE':
                simThAr_sci.append(archivo)
                obname = h[0].header['OBJECT']
                ra     = h[0].header['HIERARCH ESO INS ADC1 RA']
                delta  = h[0].header['HIERARCH ESO INS ADC1 DEC']
                try:
                    airmass= h[0].header['HIERARCH ESO TEL AIRM START']
                except:
                    airmass = -999.
                    texp   = h[0].header['EXPTIME']
                    date   = h[0].header['DATE-OBS']
                    line = "%-15s %10s %10s %8.2f %4.2f %s %8s %s\n" % (obname, ra, delta, texp, airmass, h[0].header['HIERARCH ESO DPR TYPE'], date, archivo)
                    f.write(line)
            elif h[0].header['HIERARCH ESO DPR TYPE'] == 'OBJECT,SKY' or h[0].header['HIERARCH ESO DPR TYPE'] == 'VELOC,SKY':
                simSky_sci.append(archivo)
                obname = h[0].header['OBJECT']
                ra     = h[0].header['HIERARCH ESO INS ADC1 RA']
                delta  = h[0].header['HIERARCH ESO INS ADC1 DEC']
                try:
                    airmass= h[0].header['HIERARCH ESO TEL AIRM START']
                except:
                    airmass = -999.
                    texp   = h[0].header['EXPTIME']
                    date   = h[0].header['DATE-OBS']
                    line = "%-15s %10s %10s %8.2f %4.2f %s %8s %s\n" % (obname, ra, delta, texp, airmass, h[0].header['HIERARCH ESO DPR TYPE'], date, archivo)
                    f.write(line)

            elif h[0].header['HIERARCH ESO DPR TYPE'] == 'BIAS':
                biases.append(archivo)
                mjd,mjd0 = mjd_fromheader(h)
                bias_dates.append(mjd)

            elif h[0].header['HIERARCH ESO DPR TYPE'] == 'FLAT':
                flats.append(archivo)
                mjd,mjd0 = mjd_fromheader(h)
                flat_dates.append(mjd)

            elif h[0].header['HIERARCH ESO DPR TYPE'] == 'WAVE':
                div = archivo.split('_')
                if h[0].header['HIERARCH ESO INS CALMIRR2 ID'] == lamp:
                    ThArNe_ref.append(archivo)
                    mjd, mjd0 = mjd_fromheader(h)
                    ThArNe_ref_dates.append( mjd )
                if h[0].header['HIERARCH ESO INS CALMIRR2 ID'] == lamp:

                    ThAr_Ne_ref.append(archivo)
                    mjd, mjd0 = mjd_fromheader(h)
                    ThAr_Ne_ref_dates.append( mjd )

            elif h[0].header['HIERARCH ESO DPR TYPE'] == 'DARK':
                darks.append(archivo)
                dark_times.append(h[0].header['EXPTIME'])

    f.close()
    biases, bias_dates = np.array(biases), np.array(bias_dates)
    flats, flat_dates  = np.array(flats), np.array(flat_dates)
    darks, dark_times  = np.array(darks), np.array(dark_times)
    IS = np.argsort(bias_dates)
    biases, bias_dates = biases[IS], bias_dates[IS]
    IS = np.argsort(flat_dates)
    flats, flat_dates = flats[IS], flat_dates[IS]

    return biases, flats, ThArNe_ref, ThAr_Ne_ref, simThAr_sci, simSky_sci, ThArNe_ref_dates, ThAr_Ne_ref_dates, darks, dark_times

def mjd_fromheader(h):
    """
    return modified Julian date from header
    """

    datetu = h[0].header['DATE-OBS']
    if len(datetu) < 23:
        mjd_start = h[0].header['MJD-OBS']
        mjd0 = 2400000.5
    else:
        #print(datetu)
        mjd0,mjd,i = GLOBALutils.iau_cal2jd(int(datetu[:4]),int(datetu[5:7]),int(datetu[8:10]))
        ho = int(datetu[11:13])
        mi = int(datetu[14:16])
        se = float(datetu[17:])
        ut = float(ho) + float(mi)/60.0 + float(se)/3600.0
        mjd_start = mjd + ut/24.0

    secinday = 24*3600.0
    fraction = 0.5
    texp     = h[0].header['EXPTIME'] #sec

    mjd = mjd_start + (fraction * texp) / secinday

    return mjd, mjd0

def Lines_mBack(thar, sd,  thres_rel=3, pl=False):
    """
    Given an extracted ThAr order, return a version where the background has been removed
    """


    L = np.where(sd > 0)

    # initial background estimate, simple 25% percentile on a 100-pixel filter
    bkg = np.zeros( len(sd) )
    bkg[L] = scipy.signal.order_filter(thar[L],np.ones(101),25)
    d = thar - bkg

    lines = FindLines_simple_sigma(d,sd, thres=thres_rel)
    # Now, mask these lines
    mask = np.ones( len(sd) )

    for kk in lines:
        #mask region
        if (d[kk] > 10000):
            mask[kk-3:kk+4] = 0
        elif (d[kk] > 5000):
            mask[kk-3:kk+4] = 0
        elif (d[kk] > 1000):
            mask[kk-3:kk+4] = 0
        else:
            mask[kk-3:kk+4] = 0

    # New, final background estimnate
    X = np.array( range( len( d ) ) )
    K = np.where((sd > 0) & (mask > 0))

    bkg = np.zeros( len(sd) )
    bkg_T = lowess(thar[K].astype('double'), X[K],frac=0.2,it=3,return_sorted=False)
    tck1 = scipy.interpolate.splrep(X[K],bkg_T,k=1)
    bkg[L] = scipy.interpolate.splev(X[L],tck1)

    return bkg

def FindLines_simple_sigma(d,sd,thres=3):
    """
    Given an array, find lines above a sigma-threshold
    """
    L = np.where( (d > (thres*sd)) & (d > 0) )
    lines = []
    for i in range(np.shape(L)[1]):
        j = L[0][i]
        if ((d[j] > d[j-1]) & (d[j-1] > d[j-2]) & (d[j] > d[j+1]) & (d[j+1] > d[j+2])):
            lines.append(j)

    return lines

### wavelength calibration routines ###
def sigma_clip(vec,lim=3.0):
    while True:
        med = np.median(vec)
        res = vec - med
        rms = np.sqrt(np.var(res))
        I   = np.where(np.absolute(res) < lim*rms)[0]
        if len(I) == len(vec):
            break
        else:
            vec = vec[I]
    return vec

def get_dark(time,dnames,dtimes):
    print(dnames)
    print(dtimes)
    print(time)
    if len(dnames) == 0:
        return 0.
    elif len(dnames) == 1:
        darko = pyfits.getdata(dnames[0])
        dark  = darko * float(time)/float(dtimes[0])
        print(np.median(darko),np.median(dark))
        return darko
    elif len(dnames) == 2:
        sc0 = pyfits.getdata(dnames[0]).astype('float')
        sc1 = pyfits.getdata(dnames[1]).astype('float')
        t0,t1 = float(dtimes[0]),float(dtimes[1])
        m = (sc1 - sc0) / (t1 - t0)
        n = sc1 - m*t1
        darko =  m*float(time) + n
        print(np.median(sc0),np.median(sc1))
        print(np.median(sc1*time/dtimes[1]),np.median(sc0*time/dtimes[0]))
        print(np.median(darko))
        return darko



import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import interpolate
from scipy import signal
from ceres3.utils import globalutils as GLOBALutils
import os 
import glob
from astropy.io import fits as pyfits
from scipy import ndimage
import emcee
import corner

from astropy import units as u
from astropy.time import Time
from astropy.coordinates import SkyCoord, EarthLocation, AltAz

def FileClassify(diri, log):

        # define output lists
        biases         = []
        darks              = []
        flats          = []
        flats_co           = []
        thar           = []
        thar_co            = []
        sim_sci        = []
        dar_sci            = []
        thart = []
        simt = []

        f = open(log,'w')

        #Do not consider the images specified in dir+badfiles.txt
        bad_files = []
        if os.access(diri+'bad_files.txt',os.F_OK):
                bf = open(diri+'bad_files.txt')
                linesbf = bf.readlines()
                for line in linesbf:
                        bad_files.append(diri+line[:-1])
                bf.close()
    
        all_files = glob.glob(diri+"/*fits")
        for archivo in all_files:
                #print archivo
                dump = False
                for bf in bad_files:
                        if archivo == bf:
                                dump = True
                                break
                if dump == False:
                        h = pyfits.open(archivo)
                        hd = pyfits.getheader(archivo)

                        if 'bias' in archivo:
                                biases.append(archivo)
                        elif 'dark' in archivo:
                                darks.append(archivo)
                        elif 'flat' in archivo:
                                if '1F' in archivo:
                                        flats_co.append(archivo)
                                elif '2F' in archivo:
                                        flats.append(archivo)
                        elif 'ThAr' in archivo:
                                if '1F' in archivo:
                                        thar_co.append(archivo)
                                elif '2F' in archivo:
                                        thar.append(archivo)
                                        thart.append(mjd_fromheader(h)[0])
                        else:
                                if '1F' in archivo:
                                        dar_sci.append(archivo)
                                else:
                                        sim_sci.append(archivo)
                                        simt.append(mjd_fromheader(h)[0])

                                #line = "%-15s %10s %10s %8.2f %4.2f %8s %11s %s\n" % (obname, ra, delta, texp, airmass, date, hour, archivo)
                                #f.write(line)

        thart = np.array(thart)
        simt = np.array(simt)
        f.close()
        biases, darks, flats, flats_co, thar, thar_co, sim_sci, dar_sci = \
                np.array(biases), np.array(darks), np.array(flats), np.array(flats_co), np.array(thar), \
                np.array(thar_co), np.array(sim_sci), np.array(dar_sci)
        It = np.argsort(thart)
        thar = thar[It]
        It = np.argsort(simt)
        sim_sci = sim_sci[It]
        return biases, darks, flats, flats_co, thar, thar_co, sim_sci, dar_sci

def make_flatOB(MasterFlat, c_co,exap=5):
        nord_co = len(c_co)
        flat = MasterFlat.T
        img_out = flat.copy()
        Centers = np.zeros((len(c_co),flat.shape[1]))
        ejx = np.arange(flat.shape[1])
        for o in range(nord_co):
                Centers[o,:]=np.polyval(c_co[o],ejx)
        for x in range(flat.shape[1]):
                baso = np.min(flat[:,x])
                for o in range(nord_co):
                        cen = np.around(Centers[o,x])
                        try:
                                bas = np.min(flat[int(np.around(cen-3*exap)):int(np.around(cen+3*exap)),x])
                        except:
                                bas = baso
                        img_out[int(np.around(cen-exap)):int(np.around(cen+exap+1)),x] = bas
        return img_out

def get_flatOB(MasterFlat, MasterFlat_co):
        d1 = MasterFlat[1000]
        d2 = MasterFlat_co[1000]
        ref2 = np.arange(len(d2))
        tck = scipy.interpolate.splrep(ref2,d2,k=3)
        shfts = np.arange(-6.,6.,0.001)
        ccfs = []
        for shft in shfts:
                ref = ref2 + shft
                t2 = scipy.interpolate.splev(ref,tck)
                ccfs.append(np.sum(t2*d1))
        shft = shfts[np.argmax(ccfs)]

        new_img = MasterFlat_co.copy()
        for x in range(MasterFlat_co.shape[0]):
                d = MasterFlat_co[x]
                ref = np.arange(len(d))
                tck = scipy.interpolate.splrep(ref,d,k=1)
                new_img[x] = MasterFlat[x] - scipy.interpolate.splev(ref+shft,tck)
        return new_img

def get_data(path):
    d = pyfits.getdata(path)
    d = np.fliplr(d.T)
    return d

def good_orders(coef,nord,ny,nx,ext_aperture):
    Centers = np.zeros((nord,nx))
    ejx = np.arange(nx)
    bad_inx = []
    for i in range(nord):
        Centers[i,:]=np.polyval(coef[i],ejx)
        I = np.where(Centers[i,:]+ext_aperture>ny)[0]
        if len(I)>0:
            bad_inx.append(i)
    bad_inx = np.array(bad_inx)
    im = np.min(bad_inx)
    return coef[:im], nord-len(bad_inx)

def clean_orders(c_all,data,exap=5):
    nords = len(c_all)
    medc = int(.5*data.shape[1])
    d = np.median(data[:,medc-exap:medc+exap+1],axis=1)
    #plt.plot(d)
    #plt.show()
    Centers = np.zeros((len(c_all),data.shape[1]))
    ejx = np.arange(data.shape[1])
    for i in range(len(c_all)):
        Centers[i,:]=np.polyval(c_all[i],ejx)
    ccens = Centers[:,medc]
    dist1 = ccens[-1]-ccens[-2]
    dist2 = ccens[-2]-ccens[-3]
  
    if dist1 < dist2:
        print('uno')
        i = nords - 1
    else:
        print('dos')
        i = nords - 2
    c_co, c_ob = [],[]
    while i > 0:
        print(i)
        if len(c_co) == 0:
            c_co = c_all[i]
            c_ob = c_all[i-1]
        else:
            c_co = np.vstack((c_all[i],c_co))
            c_ob = np.vstack((c_all[i-1],c_ob))
        i -= 2

        
    #for i in range(len(c_ob)):
    #    Centers[i,:]=np.polyval(c_ob[i],ejx)
    #   plt.plot(ejx,Centers[i,:],'r')
    #for i in range(len(c_co)):
    #    Centers[i,:]=np.polyval(c_co[i],ejx)
    #   plt.plot(ejx,Centers[i,:],'b')
    #plt.show()
    return c_ob, c_co, len(c_ob), len(c_co)

def mjd_fromheader(h):
    """
    return modified Julian date from header
    """
    datetu = h[0].header['DATE-OBS'][:10]
    ut     = h[0].header['DATE-OBS'][11:]
    mjd0,mjd,i = GLOBALutils.iau_cal2jd(int(datetu[0:4]),int(datetu[5:7]),int(datetu[8:10]))
    ut = float(ut[:2])+ float(ut[3:5])/60. + float(ut[6:])/3600.
    mjd_start = mjd + ut/24.0

    #print 'Warning!!! adding 5 hrs to comute MJD due to problem in header! CHECK in future!!'
    #mjd_start += 5./24.


    secinday = 24*3600.0
    fraction = 0.5
    texp     = h[0].header['EXPTIME'] #sec

    #print 'Warning!!! assuming that the provided date is at the end of the exposure!!! Check in the future!!'
    #mjd = mjd_start - (fraction * texp) / secinday

    mjd = mjd_start + (fraction * texp) / secinday

    return mjd, mjd0

def scat_flat(d,c_ob,c_co):
    Centers_co = np.zeros((len(c_co),d.shape[1]))
    Centers_ob = np.zeros((len(c_ob),d.shape[1]))
    ejx = np.arange(d.shape[1])
    ejy = np.arange(d.shape[0])
    for i in range(len(c_co)):
        Centers_co[i,:]=np.around(np.polyval(c_co[i],ejx)).astype('int')
        Centers_ob[i,:]=np.around(np.polyval(c_ob[i],ejx)).astype('int')
    i = 0
    while i < d.shape[1]:
        line = d[:,i]
        refmins = []
        valmins = []
        for j in range(len(Centers_co)):
            p1 = Centers_ob[j,i]
            p2 = Centers_co[j,i]
            minv = line[p1-10:p2+10]
            refv = ejy[p1-10:p2+10]
            im = np.argmin(minv)
            refmins.append(refv[im])
            valmins.append(minv[im])
        refmins,valmis = np.array(refmins),np.array(valmins)
        #plt.plot(line)
        #plt.plot(refmins,valmins,'ro')
        #plt.show()
        #print gfd
            
def get_scat(sc,ps,binning):
        binning = float(binning)
        #print sc.shape,ps.shape
        ejex = np.arange(sc.shape[1])
        bac = sc.copy()
        for col in range(sc.shape[0]):
                #col = 1000
                #plt.plot(sc[col])
                #plt.show()
                #print gfds
                J = np.where(ps[:,col]==0)[0]
                Z = np.zeros((sc.shape[1]))
                Z[J] = 1.
                #plt.plot(sc[col])
                #plt.plot(sc[col]*Z,linewidth=2.0)
                #ncol = scipy.signal.medfilt(sc[col],11)
                Z2 = np.append(Z[-1],Z[:-1])
                I = np.where((Z!=0) & (Z2==0))[0]
                J = np.where((Z2!=0) & (Z==0))[0]
                J = J[1:]
                I = I[:-1]
                points = []
                vals = []
                for i in range(len(I)):
                        #plt.plot(numpy.mean(ejex[I[i]:J[i]]),numpy.median(sc[2000,I[i]:J[i]]),'ro')
                        points.append(np.mean(ejex[I[i]:J[i]]))
                        vals.append(np.median(sc[col,I[i]:J[i]]))
                points,vals = np.array(points),np.array(vals)
                #pi = 0
                #npoints,nvals = [],[]
                #while pi < len(points)-1:
                #       if vals[pi]>vals[pi+1]:
                #               npoints.append(points[pi+1])
                #               nvals.append(vals[pi+1])
                #       else:
                #               npoints.append(points[pi])
                #               nvals.append(vals[pi])
                #       pi+=2
                #points,vals = np.array(npoints),np.array(nvals)
                
                #vals = scipy.signal.medfilt(vals,3)
                #plt.plot(points,vals,'ro')
                #plt.plot(points,vals,'r')
                #plt.plot(points,scipy.signal.medfilt(vals,3),'k')
                #plt.show()
                F = np.where(vals < 10000)[0]
                vals = vals[F]
                points = points[F]
                tck = interpolate.splrep(points,vals,k=1)
                scat = interpolate.splev(ejex,tck)
                
                scat[:I[0]] = scat[I[0]]
                #scat[J[-1]:] = 0.
                #plt.plot(scat)
                bac[col] = scat
                #plt.plot(ejex,scat,'r')
                #plt.show()
        bacm = signal.medfilt2d(bac,[51,1])
        #plt.plot(bac[:,1000])
        #plt.plot(sc[1000])
        #plt.plot(bacm[1000])
        #plt.show()
        return bacm

def get_name(arch):
        obname = pyfits.getheader(arch)['OBJECT']
        if 'HD10700' in arch:
                obname = 'HD10700'
        return obname

            
def Initial_Wav_Calibration(filename,spec,order,wei, porder=3, rmsmax=75, minlines=10, FixEnds=True, \
                            Dump_Argon=False, Dump_AllLines=False, Cheby=False, rough_shift = 0.0,del_width=5.0, \
                            binning=1,line_width=4, fact=1,do_xc=True,sigmai=0.7,pixelization=False,fibre_ap=2.3):

        f = open(filename).readlines()
        bad_indices = np.array([])
        pixel_centers = np.array([])
        wavelengths   = np.array([])
        sigmas        = np.array([])
        centroids     = np.array([])
        intensities   = np.array([])

        delta=0.
        N_l = 0
        out = []
        for line in f:
            if line[0]!='#':
                w = line.split()
                nlines = int(w[0])
                pix = float(w[1])*fact/float(binning) + delta + rough_shift
                wav = float(w[2])
                if pix > 20 and pix < len(spec)-20:
                        sigma = sigmai * fact / float(binning)
                        X = np.arange(int(np.around(pix))-5,int(np.around(pix))+6)
                        Y = spec[X]
                        guess = np.array([pix-0.5*fibre_ap,np.max(Y),sigmai])
                        p1 = FitFideosCompProf(X, Y, guess, fibre_ap)

                        pixel_centers = np.append(pixel_centers,p1[0])
                        sigmas        = np.append(sigmas,p1[2])
                        wavelengths   = np.append(wavelengths,wav)
                        intensities   = np.append(intensities,p1[1])
                        centroids     = np.append(centroids, p1[0])
                        N_l += 1

        #plt.show()
        pixel_centers2 = np.around(pixel_centers).astype('int')
        I = np.where((pixel_centers2>=0) & (pixel_centers2<len(spec)))
        pixel_centers2 = pixel_centers2[I]
        #plt.plot(pixel_centers2,spec[pixel_centers2],'go')
        #plt.show()
        #print gfd

        #I = np.where((pixel_centers>0) & (pixel_centers<2048))[0]
        #plt.plot(np.around(pixel_centers[I]).astype('int'),spec[np.around(pixel_centers[I]).astype('int')],'ro')
        #plt.show()
        I1 = np.where(pixel_centers<50)[0]
        I2 = np.where(pixel_centers>len(spec)-50)[0]
        II = np.hstack((I1,I2))
        bad_indices = np.hstack((np.array(bad_indices),II))
        bad_indices = list(np.unique(bad_indices))
        # now, do the polynomial fit, rejecting some lines until RMS is below rmsmax
        I = list(range( N_l ))

        for bi in bad_indices:
                I.remove( bi )
                N_l -= 1

        if (Cheby):
                coeffs_pix2wav   = Cheby_Fit(pixel_centers[I], wavelengths[I], porder,len(spec))   
                coeffs_pix2sigma = Cheby_Fit(pixel_centers[I], sigmas[I], porder,len(spec))
        else:
                coeffs_pix2wav   = np.polyfit(pixel_centers[I], wavelengths[I], porder)    
                coeffs_pix2sigma = np.polyfit(pixel_centers[I], sigmas[I], porder)

        rmsms, residuals = rms_ms(coeffs_pix2wav, pixel_centers[I], wavelengths[I], len(spec), Cheby=Cheby)

        if (FixEnds):
                minI = np.min( I ) + 1
                maxI = np.max( I ) - 1
        else:
                minI = np.min( I )
                maxI = np.max( I )
        #if order==26:
        #           plt.plot(pixel_centers[I],residuals,'ro')
        #           plt.plot([0,4096],[0,0])
        #plt.plot(np.arange(4096),Cheby_eval(coeffs_pix2wav,np.arange(4096),len(spec)))
        #plt.show()
        #print dfgh
        count = 0
        while ((N_l > minlines) and (rmsms > rmsmax)):
                rmsms, residuals = rms_ms(coeffs_pix2wav, pixel_centers[I], wavelengths[I], len(spec), Cheby=Cheby)
                index_worst = np.argmax( np.absolute(residuals) )
                I.pop( index_worst)
                N_l -= 1
                if (Cheby):
                        coeffs_pix2wav   = Cheby_Fit(pixel_centers[I], wavelengths[I], porder,len(spec))   
                        coeffs_pix2sigma = Cheby_Fit(pixel_centers[I], sigmas[I], porder,len(spec))
                else:
                        coeffs_pix2wav   = np.polyfit(pixel_centers[I], wavelengths[I], porder)    
                        coeffs_pix2sigma = np.polyfit(pixel_centers[I], sigmas[I], porder)
                count +=1

        rmsms, residuals = rms_ms(coeffs_pix2wav, pixel_centers[I], wavelengths[I], len(spec), Cheby=Cheby)    

        pci = np.around(pixel_centers).astype('int')
        #plt.plot(spec)
        #plt.plot(pci,spec[pci],'ro')
        #plt.plot(pci[I],spec[pci[I]],'bo')
        #plt.show()
        #plt.plot(wavelengths[I],residuals,'ro')
        #plt.show()
        #print "RMS is ", rmsms, "using ", N_l, " lines at indices ", I
        #plt.plot(pixel_centers[I],wavelengths[I],'ro')
        #if order == 26:
        #           plt.plot(pixel_centers[I],residuals-0.1,'bo')
        #           plt.plot([0,4096],[-0.1,-0.1])
        #           #plt.plot(np.arange(4096),Cheby_eval(coeffs_pix2wav,np.arange(4096),len(spec)))
        #           plt.show()
        #print order, len(pixel_centers), len(I)
        return coeffs_pix2wav, coeffs_pix2sigma, pixel_centers[I], wavelengths[I], \
        rmsms, residuals, centroids[I], sigmas[I], intensities[I]

def rms_ms(coeffs_pix2wav, pixel_centers, wavelengths, npix, Cheby=False):
    " Returns rms deviation of best fit in m/s"

    if (Cheby):
        residuals = Cheby_eval(coeffs_pix2wav,pixel_centers,npix) - wavelengths
        central_wav = 0.5 * (Cheby_eval(coeffs_pix2wav,50.,npix) + Cheby_eval(coeffs_pix2wav,npix-50,npix))
    else:
        residuals = np.polyval(coeffs_pix2wav,pixel_centers) - wavelengths
        central_wav = 0.5 * (np.polyval(coeffs_pix2wav,50.) + np.polyval(coeffs_pix2wav,npix-50))

    rms_ms = np.sqrt( np.var( residuals ) ) * 299792458.0 / central_wav
    
    return rms_ms, residuals

def fit2d(sc,trace,filename,spec,order,wei, porder=3, rmsmax=75, minlines=10, FixEnds=True, \
                            Dump_Argon=False, Dump_AllLines=False, Cheby=False, rough_shift = 0.0,del_width=5.0, \
                            binning=1,line_width=4, fact=1,do_xc=True,sigmai=0.8,pixelization=False):

        f = open(filename).readlines()

        pixel_centers = np.array([])
        wavelengths   = np.array([])
        sigmas        = np.array([])
        centroids     = np.array([])
        intensities   = np.array([])

        delta=0.
        N_l = 0
        out = []
        for line in f:
            if line[0]!='#':
                w = line.split()
                nlines = int(w[0])
                pix = float(w[1])*fact/float(binning) + delta + rough_shift
                wav = float(w[2])
                if pix > 20 and pix < len(spec)-20:
                        #X = np.array(range(pix-line_width,pix+line_width+1))
                        #Y = spec[pix-line_width:pix+line_width+1]
                        sigma = sigmai * fact / float(binning)
                        #p1 = FitFideosCompProf(X, Y, B, mu[0], sigma[0])
                        #print trace
                        y1 = 2064 - np.polyval(trace,2048.-pix)
                        #print '#',2048-pix,y1
                        
                        X = np.arange(int(np.around(2048.-pix))-5,int(np.around(2048-pix))+5)
                        Y = np.arange(int(np.around(y1))-5,int(np.around(y1))+6)

                        F = sc.T
                        F = F[X,:]
                        F = F[:,Y]
                        B1 = F[0]
                        B2 = F[-1]
                        B3 = F[:,0]
                        B4 = F[:,-1]
                        BAC = np.median(np.hstack((B1,B2,B3,B4)))
                        F = F - BAC
                        Ferr = np.sqrt(np.sqrt(np.absolute(F))**2 + 9.)
                        #pars = np.array([1.8128e+03, 1.7167e+03, 7.500e+03, 2.3, 6.15])
                        #model = FideosCompProf(pars,X,Y)
                        #model2 = FideosCompProf(np.array([2048-pix, y1,F.max()]),X,Y)
                        #for i in range(F.shape[1]):
                        #       plt.errorbar(X,F[:,i],yerr=Ferr[:,i],fmt='b')
                        #       plt.plot(X,model[:,i],'r')
                        #       plt.plot(Y,model2[i],'k')
                        #plt.show()

                        #print X,Y,pix,y1
                        #p1 = FitFideos2DProf(X, Y, F, 2048-pix, y1)
                        guess = np.array([2048-pix, y1,F.max(),2.,10.])
                        ndim = len(guess)
                        nwalkers=100
                        pos = [guess + np.random.randn(ndim)*np.array([2,2,500.,2.,4.]) for i in range(nwalkers)]
                        sampler = emcee.EnsembleSampler(nwalkers, ndim, lnprob, args=(X,Y,F,Ferr))
                        sampler.run_mcmc(pos, 500)
                        samples = sampler.chain[:, 50:, :].reshape((-1, ndim))
                        #print samples.shape
                        print(np.median(samples,axis=0))
                        if len(out) == 0:
                                out = np.median(samples,axis=0)
                        else:
                                out = np.vstack((out,np.median(samples,axis=0)))
                        #fig = corner.corner(samples, labels=["$Xpos$", "$Ypos$", "Amp", 'Fibwidth', 'Inst'])
                        #plt.show()
        return out

def rms_ms(coeffs_pix2wav, pixel_centers, wavelengths, npix, Cheby=False):
    " Returns rms deviation of best fit in m/s"

    if (Cheby):
        residuals = Cheby_eval(coeffs_pix2wav,pixel_centers,npix) - wavelengths
        central_wav = 0.5 * (Cheby_eval(coeffs_pix2wav,50.,npix) + Cheby_eval(coeffs_pix2wav,npix-50,npix))
    else:
        residuals = np.polyval(coeffs_pix2wav,pixel_centers) - wavelengths
        central_wav = 0.5 * (np.polyval(coeffs_pix2wav,50.) + np.polyval(coeffs_pix2wav,npix-50))

    rms_ms = np.sqrt( np.var( residuals ) ) * 299792458.0 / central_wav
    
    return rms_ms, residuals

def Cheby_Fit(x,y,order,npix):
    """
    Fits Chebyshev polynomials to y as a function of x

    """
    med = .5*npix
    #normalize x
    x_norm = (x-med) / med

    def fitfunc(p,chebs,order):
        ret_val = 0.0
        for i in range(0,order+1):
            ret_val += p[order-i]*chebs[i]
        return ret_val
    errfunc = lambda p,chebs,y,order: np.ravel( (fitfunc(p,chebs,order)-y) )
    
    def get_chebs(x,order):
        chebs = []
        for i in range(0,order+1):
            chebs.append( scipy.special.chebyt(i)(x) )
        return chebs

    p0 = np.zeros( order + 1 )
    p0[order] = np.mean( y )
    chebs = get_chebs( x_norm, order)
    p1, success = scipy.optimize.leastsq(errfunc, p0, args=(chebs, y, order))
    return p1

def Cheby_eval(p,x,npix):
    """
    evaluates Chebyshev polynomial fit at x given best-fit parameters p
    """
    med = .5*npix
    x_norm = (x-med) / med
    order = len(p) - 1
    ret_val = 0.0
    for i in range(order + 1):
        ret_val += p[order - i]*scipy.special.chebyt(i)(x_norm)
    
    return ret_val

def LineFit_SingleSigma(X, Y, B, mu, sigma, weight,pixelization=False):
    """
    This function fits a series of Gaussians simultaneously, given a set
    of input pixels, sigmas, and intensities
    
    Sigma is the same for all lines
    
    Returns (mu_i, sigm_i, int_i), i=1,.,n where n is the number of components
    """

    # get number of components to be fit
    n = len(mu)

    def fitfunc(p, x, n):

        if pixelization:
            lxo = len(x)
            xo = x.copy()
            x = np.arange(x[0]-0.5,x[-1]+0.5,0.01)

        ret = np.zeros(len(x))
        for i in range(n):
            ret += ( p[i*2+1] * IntGaussian(x,p[i*2+2],p[0]) )

        if pixelization:
            ret = ret.reshape((lxo,100))
            ret = np.mean(ret,axis=1)


        return ret
    errfunc = lambda p,x,n,y,weight: np.ravel( (fitfunc(p,x,n)-y)*weight )

    # Build error array
    #err = np.sqrt((B+Y)/gain + readnoise**2)
    
    # Build input parameters list
    # length of inout parameter list: 3 * n
    p0 = np.zeros(2*n+1)
    p0[0] = sigma[0]
    for i in range(n):
        p0[i*2+1] = Y[int(np.round(mu[i])-X[0])]
        p0[i*2+2] = mu[i]

    # perform fit
    #plt.plot(X,Y,'b')
    
    p1, success = scipy.optimize.leastsq(errfunc,p0, args=(X,n,Y-B, weight))
    #plt.plot(X,fitfunc(p1,X,n),'r')
    # build output consistent with LineFit
    p_output = np.zeros(3*n)
    for i in range(n):
        p_output[i*3]   = p1[i*2 + 1]
        p_output[i*3+1] = p1[i*2 + 2]
        p_output[i*3+2] = p1[0]

    return p_output, success  

def TruncGaussian(x,y,x0,y0,dev):
        if x > x0:
                return 0
        elif x < x0 - 3*dev:
                return 0
        c1 = (x - x0)**2/(2.*dev*dev)
        c2 = (y - y0)**2/(2.*dev*dev)
        z = np.exp(-(c1+c2))
        return z

def HalfFiber(x,y,x0,y0,lim):
        if x > x0:
                return 0.
        if np.sqrt((x-x0)**2 + (y-y0)**2)<lim:
                return 1.
        else:
                return 0.

def HalfFiberPos(x,y,x0,y0,lim):
        if x < x0:
                return 0.
        if np.sqrt((x-x0)**2 + (y-y0)**2)<lim:
                return 1.
        else:
                return 0.

def FitFideosCompProf(X, Y, p0, fibre_ap):

    def AnaliticProfile1D(p,x,R):
        space = 100.
        xt    = np.arange(x[0],x[-1]+1,1./space)
        x0 = xt - p[0]
        J = np.where((R**2-x0**2>=0) & (x0>=0))[0]
        out = np.zeros(len(x0))
        out[J] = 2*np.sqrt(R**2-x0[J]**2)
        out = scipy.ndimage.filters.gaussian_filter(out, p[2]*space)
        out /= np.max(out)
        out = p[1]*out
        out = np.mean(out.reshape((int(len(out)/space),int(space))),axis=1)
        return out
    errfunc = lambda p,X,Y,fibre_ap: np.ravel( (AnaliticProfile1D(p,X,fibre_ap)-Y) )

    p1, success = scipy.optimize.leastsq(errfunc,p0, args=(X,Y,fibre_ap))
    #plt.plot(X,Y)
    #plt.plot(X,AnaliticProfile1D(p1,X,fibre_ap))
    #plt.plot(X,AnaliticProfile1D(p0,X,fibre_ap))
    return p1

def FideosCompProf(p,X,Y):
            Xt = np.arange(X[0]-0.5,X[-1]+0.6,0.1)
            Yt = np.arange(Y[0]-0.5,Y[-1]+0.6,0.1)
            mat = np.zeros((len(Xt),len(Yt)))
            for x in np.arange(len(Xt)):
                for y in  np.arange(len(Yt)):
                    mat[x,y] = HalfFiber(Xt[x],Yt[y],p[0],p[1],p[3])
            fin2 = scipy.ndimage.filters.gaussian_filter(mat, p[4])
            fin2 /= np.max(fin2)
            fin2 = p[2]*fin2
            
            mat2 = np.zeros((len(X),len(Y)))
            for x in np.arange(len(X)):
                I = np.where((Xt>=X[x]-0.5)&(Xt<X[x]+0.4999))[0]
                temp = fin2[I]
                for y in  np.arange(len(Y)):
                        J = np.where((Yt>=Y[y]-0.5)&(Yt<Y[y]+0.4999))[0]
                        mat2[x,y] = np.mean(temp[:,J])
            return mat2


def FitFideos2DProf(X, Y, F, x1, y1):
        def FideosCompProf(p,X,Y):
            Xt = np.arange(X[0]-0.5,X[-1]+0.6,0.1)
            Yt = np.arange(Y[0]-0.5,Y[-1]+0.6,0.1)
            mat = np.zeros((len(Xt),len(Yt)))
            for x in np.arange(len(Xt)):
                for y in  np.arange(len(Yt)):
                    mat[x,y] = HalfFiber(Xt[x],Yt[y],p[0],p[1],2.)
            fin2 = scipy.ndimage.filters.gaussian_filter(mat, 5.)
            fin2 /= np.max(fin2)
            fin2 = p[2]*fin2
            
            mat2 = np.zeros((len(X),len(Y)))
            for x in np.arange(len(X)):
                I = np.where((Xt>=X[x]-0.5)&(Xt<X[x]+0.4999))[0]
                temp = fin2[I]
                for y in  np.arange(len(Y)):
                        J = np.where((Yt>=Y[y]-0.5)&(Yt<Y[y]+0.4999))[0]
                        mat2[x,y] = np.sum(temp[:,J])
            return mat2
        errfunc = lambda p,X,Y,F: np.ravel( (FideosCompProf(p,X,Y)-F) )
        
        p0 = np.array([x1,y1,np.max(F)])
        p1, success = scipy.optimize.leastsq(errfunc,p0, args=(X,Y,F))
        print(p0)
        print(p1)
        plt.imshow(F)
        plt.show()
        plt.imshow(FideosCompProf(p1,X,Y))
        plt.show()
        return p1


def lnlike(theta, X,Y,F, Ferr):
        model = FideosCompProf(theta,X,Y)
        inv_sigma2 = 1.0/(Ferr**2)
        #ret = -np.log(2*np.pi) + np.log(np.sum(np.exp(-0.5*((y-model)/yerr)**2)/yerr))
        ret = -0.5*(np.sum(inv_sigma2*(F-model)**2 - np.log(inv_sigma2)))
        #for i in range(len(F)):
        #       plt.errorbar(Y,F[i],yerr=Ferr[i],fmt='b')
        #for j in model:
        #       plt.plot(Y,j,'r')
        #plt.show()
        #print theta, ret
        if np.isnan(ret):
                return -np.inf
        else:
                return ret


def lnprior(theta,X,Y):
        if X[0] < theta[0] < X[-1] and Y[0] < theta[1] < Y[-1] and 0 < theta[2] < 70000 and 0.1 < theta[3] < 5 and  0.1 < theta[4] < 20:
            return 0.0
        return -np.inf

def lnprob(theta, X,Y,F,Ferr):
    lp = lnprior(theta,X,Y)
    if not np.isfinite(lp):
        return -np.inf
    return lp + lnlike(theta,X,Y,F,Ferr)

def AnaliticProfile1D(p,x,R=2.3):
    space = 100.
    xt    = np.arange(x[0],x[-1]+1,1./space)
    x0 = xt - p[0]
    J = np.where((R**2-x0**2>=0) & (x0>=0))[0]
    out = np.zeros(len(x0))
    out[J] = 2*np.sqrt(R**2-x0[J]**2)
    out = scipy.ndimage.filters.gaussian_filter(out, p[2]*space)
    out /= np.max(out)
    out = p[1]*out
    out = np.mean(out.reshape((len(out)/space,space)),axis=1)
    return out


def get_airmass(ra,dec,lat,lon,alt,dateobs):

        target = SkyCoord(ra,dec, unit="deg")  
        obs    = EarthLocation(lat=lat*u.deg, lon=lon*u.deg, height=alt*u.m)
        #utcoffset = -4*u.hour  # Eastern Daylight Time
        time = Time(dateobs)
        altaz = target.transform_to(AltAz(obstime=time,location=obs))  

        return altaz.secz

def compute_RON(mbias,biases,gain=1.):
        stdevs = []
        for fbias in biases:
                bias = pyfits.getdata(fbias)
                tbias = bias-mbias
                stdevs.append(np.sqrt(np.var(tbias)))
        stdevs = np.array(stdevs)
        return np.median(stdevs)*gain
                
def fill_list_header(hdu,ofiles):
    for i in range(len(ofiles)):
        hdu = GLOBALutils.update_header(hdu,'RAW'+str(i),ofiles[i].split('/')[-1])
    return hdu
